#ifndef LIBMPEG2_H
#define LIBMPEG2_H

#ifdef __cplusplus
extern "C" {
#endif

#include "mpeg2private.h"

// Check for compatibility.  Return 1 if compatible.
int mpeg2_check_sig(char *path);

// Open the MPEG2 stream.
mpeg2_t* mpeg2_open(char *path);
int mpeg2_close(mpeg2_t *file);

// Query the MPEG2 stream.
int mpeg2_has_audio(mpeg2_t *file);
int mpeg2_audio_channels(mpeg2_t *file);
int mpeg2_sample_rate(mpeg2_t *file);
long mpeg2_audio_samples(mpeg2_t *file);            // Total length
int mpeg2_set_sample(mpeg2_t *file, long sample);   // Seek
long mpeg2_get_sample(mpeg2_t *file);               // Tell current position

// Read a PCM buffer of audio from 1 channel and advance the position.
// Return a 1 if error.
int mpeg2_read_audio(mpeg2_t *file, float *output_f, short *output_i, int channel, long samples);
// Read the next compressed audio chunk.  Store the size in size and return a 
// 1 if error.
int mpeg2_read_audio_chunk(mpeg2_t *file, unsigned char *output, long *size, long max_size);

int mpeg2_has_video(mpeg2_t *file);
int mpeg2_video_width(mpeg2_t *file);
int mpeg2_video_height(mpeg2_t *file);
float mpeg2_frame_rate(mpeg2_t *file);
long mpeg2_video_frames(mpeg2_t *file);             // Total length
int mpeg2_set_frame(mpeg2_t *file, long frame);     // Seek
long mpeg2_get_frame(mpeg2_t *file);                // Tell current position

// Read a frame.
int mpeg2_read_frame(mpeg2_t *file, unsigned char *output);
// Read the next compressed frame including headers.
// Store the size in size and return a 1 if error.
int mpeg2_read_video_chunk(mpeg2_t *file, unsigned char *output, long *size, long max_size);

#ifdef __cplusplus
}
#endif

#endif
