#include "../mpeg2private.inc"
#include "mpeg2video.h"
#include "vlc.h"

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_MMX
static unsigned long long MMX_128 = 0x80008000800080LL;
#endif

int mpeg2video_get_cbp(mpeg2video_t *video)
{
  	int code;

  	if((code = mpeg2bits_showbits(video->vstream, 9)) >= 128)
	{
    	code >>= 4;
    	mpeg2bits_flushbits(video->vstream, mpeg2_CBPtab0[code].len);
    	return mpeg2_CBPtab0[code].val;
  	}

  	if(code >= 8)
	{
    	code >>= 1;
    	mpeg2bits_flushbits(video->vstream, mpeg2_CBPtab1[code].len);
    	return mpeg2_CBPtab1[code].val;
  	}

  	if(code < 1)
	{
    	fprintf(stderr,"mpeg2video_get_cbp: invalid coded_block_pattern code\n");
    	video->fault = 1;
    	return 0;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_CBPtab2[code].len);
  	return mpeg2_CBPtab2[code].val;
}


int mpeg2video_getdclum(mpeg2video_t *video)
{
	int code, size, val;

/* decode length */
	code = mpeg2bits_showbits(video->vstream, 5);

	if(code < 31)
	{
    	size = mpeg2_DClumtab0[code].val;
    	mpeg2bits_flushbits(video->vstream, mpeg2_DClumtab0[code].len);
	}
	else 
	{
    	code = mpeg2bits_showbits(video->vstream, 9) - 0x1f0;
    	size = mpeg2_DClumtab1[code].val;
    	mpeg2bits_flushbits(video->vstream, mpeg2_DClumtab1[code].len);
	}

	if(size == 0) val = 0;
	else 
	{
    	val = mpeg2bits_getbits(video->vstream, size);
    	if((val & (1 << (size - 1))) == 0)  val -= (1 << size) - 1;
	}

	return val;
}


int mpeg2video_getdcchrom(mpeg2video_t *video)
{
	int code, size, val;

/* decode length */
	code = mpeg2bits_showbits(video->vstream, 5);

	if(code<31)
	{
    	size = mpeg2_DCchromtab0[code].val;
    	mpeg2bits_flushbits(video->vstream, mpeg2_DCchromtab0[code].len);
	}
	else 
	{
    	code = mpeg2bits_showbits(video->vstream, 10) - 0x3e0;
    	size = mpeg2_DCchromtab1[code].val;
    	mpeg2bits_flushbits(video->vstream, mpeg2_DCchromtab1[code].len);
	}

	if(size == 0) val = 0;
	else 
	{
      val = mpeg2bits_getbits(video->vstream, size);
      if((val & (1 << (size - 1))) == 0) val -= (1 << size) - 1;
	}

	return val;
}


// decode one intra coded MPEG-1 block

int mpeg2video_getintrablock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp, int dc_dct_pred[])
{
	register int val, i, j, sign;
	unsigned int code;
	mpeg2_DCTtab_t *tab = 0;
	short *bp = layerdata->block[comp];

/* decode DC coefficients */
  	if(comp < 4)         
  		bp[0] = (dc_dct_pred[0] += mpeg2video_getdclum(video)) << 3;
  	else 
  	if(comp == 4)   
  		bp[0] = (dc_dct_pred[1] += mpeg2video_getdcchrom(video)) << 3;
	else                
  		bp[0] = (dc_dct_pred[2] += mpeg2video_getdcchrom(video)) << 3;

#ifdef HAVE_MMX
  	bp[0] <<= 4;
#endif

  	if(video->fault) return 1;

/* decode AC coefficients */
  	for(i = 1; ; i++)
	{
    	code = mpeg2bits_showbits(video->vstream, 32);
    	code >>= 16;
    	if(code >= 16384) 
			tab = &mpeg2_DCTtabnext[(code >> 12) - 4];
    	else 
		if(code >= 1024) tab = &mpeg2_DCTtab0[(code >> 8) - 4];
    	else 
		if(code >= 512) tab = &mpeg2_DCTtab1[(code >> 6) - 8];
    	else 
		if(code >= 256) tab = &mpeg2_DCTtab2[(code >> 4) - 16];
    	else 
		if(code >= 128) tab = &mpeg2_DCTtab3[(code >> 3) - 16];
    	else 
		if(code >= 64) tab = &mpeg2_DCTtab4[(code >> 2) - 16];
    	else 
		if(code >= 32) tab = &mpeg2_DCTtab5[(code >> 1) - 16];
    	else 
		if(code >= 16) tab = &mpeg2_DCTtab6[code - 16];
    	else 
		{
    	  	fprintf(stderr, "mpeg2video_getintrablock: invalid Huffman code\n");
    	  	video->fault = 1;
    	  	return;
    	}

    	mpeg2bits_flushbits(video->vstream, tab->len);

    	if(tab->run == 64) break;  // end_of_block

    	if(tab->run == 65)
		{         
// escape
    		i += mpeg2bits_getbits(video->vstream, 6);

    		if((val = mpeg2bits_getbits(video->vstream, 8)) == 0) 
				val = mpeg2bits_getbits(video->vstream, 8);
    		else 
			if(val == 128)         
				val = mpeg2bits_getbits(video->vstream, 8) - 256;
    		else 
			if(val > 128)          
				val -= 256;

    		if((sign = (val < 0)) != 0) val= -val;
    	}
    	else 
		{
    		i += tab->run;
    		val = tab->level;
    		sign = mpeg2bits_getbits(video->vstream, 1);
    	}

    	j = mpeg2_zig_zag_scan[i];

#ifdef HAVE_MMX
    	val = (val * layerdata->quant_scale * layerdata->intra_quantizer_matrix[j]) << 1;
    	val = (val - 16) | 16;
#else
    	val = (val * layerdata->quant_scale * layerdata->intra_quantizer_matrix[j]) >> 3;
    	val = (val - 1) | 1;
#endif

    	bp[j] = sign ? -val : val;
	}

	if(j != 0) 
	{
/* not a layerdata->sparse matrix ! */
       layerdata->sparse[comp] = 0;
	}
	return 0;
}


// decode one non-intra coded MPEG-1 block

int mpeg2video_getinterblock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp)
{
	register int val, i, j, sign;
	unsigned int code;
	mpeg2_DCTtab_t *tab; 
	short *bp = layerdata->block[comp];

/* decode AC coefficients */
	for(i = 0; ; i++)
	{
    	code = mpeg2bits_showbits(video->vstream, 32);
    	code >>= 16;
    	if(code >= 16384)
		{
    	    if(i == 0) 
				tab = &mpeg2_DCTtabfirst[(code >> 12) - 4];
    	    else      
				tab = &mpeg2_DCTtabnext[(code >> 12) - 4];
    	}
    	else 
		if(code >= 1024) tab = &mpeg2_DCTtab0[(code >> 8) - 4];
    	else 
		if(code >= 512)  tab = &mpeg2_DCTtab1[(code >> 6) - 8];
    	else 
		if(code >= 256)  tab = &mpeg2_DCTtab2[(code >> 4) - 16];
    	else 
		if(code >= 128)  tab = &mpeg2_DCTtab3[(code >> 3) - 16];
    	else 
		if(code >= 64)   tab = &mpeg2_DCTtab4[(code >> 2) - 16];
    	else 
		if(code >= 32)   tab = &mpeg2_DCTtab5[(code >> 1) - 16];
    	else 
		if(code >= 16)   tab = &mpeg2_DCTtab6[code - 16];
    	else 
		{
    		fprintf(stderr,"mpeg2video_getinterblock: invalid Huffman code\n");
    		video->fault = 1;
    		return 1;
    	}

    	mpeg2bits_flushbits(video->vstream, tab->len);

// end of block
    	if(tab->run == 64)
    	   break;   

    	if(tab->run == 65)
		{          
// escape 
    		i += mpeg2bits_getbits(video->vstream, 6);
    		if((val = mpeg2bits_getbits(video->vstream, 8)) == 0) 
				val = mpeg2bits_getbits(video->vstream, 8);
    		else 
			if(val == 128)  
				val = mpeg2bits_getbits(video->vstream, 8) - 256;
    		else 
			if(val > 128) 
				val -= 256;

    		if((sign = (val < 0)) != 0) val = -val;
    	}
    	else 
		{
    		i += tab->run;
    		val = tab->level;
    		sign = mpeg2bits_getbits(video->vstream, 1);
    	}

    	j = mpeg2_zig_zag_scan[i];
#ifdef HAVE_MMX
    	val = (((val << 1)+1) * layerdata->quant_scale * layerdata->non_intra_quantizer_matrix[j]);
    	val = (val - 16) | 16;
#else
    	val = (((val << 1)+1) * layerdata->quant_scale * layerdata->non_intra_quantizer_matrix[j]) >> 4;
    	val = (val - 1) | 1;
#endif
    	bp[j] = sign ? -val : val;
	}

	if(j != 0) 
	{
/* not a sparse matrix ! */
       layerdata->sparse[comp] = 0;
	}
	return 0;
}


// decode one intra coded MPEG-2 block

int mpeg2video_getmpg2intrablock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp, int dc_dct_pred[])
{
	register int val, i, j, sign, nc;
	unsigned int code;
	mpeg2_DCTtab_t *tab;
	short *bp;
	int *qmat;

/* with data partitioning, data always goes to base layer */
  	bp = layerdata->block[comp];

  	qmat = (comp < 4 || video->chroma_format == CHROMA420)
         ? layerdata->intra_quantizer_matrix
         : layerdata->chroma_intra_quantizer_matrix;

/* decode DC coefficients */
	if(comp < 4)           
		val = (dc_dct_pred[0] += mpeg2video_getdclum(video));
	else 
	if((comp & 1) == 0) 
		val = (dc_dct_pred[1] += mpeg2video_getdcchrom(video));
	else                  
		val = (dc_dct_pred[2] += mpeg2video_getdcchrom(video));

  	if(video->fault) return;
#ifdef HAVE_MMX
  	bp[0] = val << (7 - video->dc_prec);
#else
  	bp[0] = val << (3 - video->dc_prec);
#endif
  	nc = 0;

/* decode AC coefficients */
  	for(i = 1; ; i++)
	{
    	code = mpeg2bits_showbits(video->vstream, 16);
    	if(code >= 16384 && !video->intravlc)
			tab = &mpeg2_DCTtabnext[(code >> 12) - 4];
    	else 
		if(code >= 1024)
		{
    		if(video->intravlc) 
				tab = &mpeg2_DCTtab0a[(code >> 8) - 4];
    		else 
				tab = &mpeg2_DCTtab0[(code >> 8) - 4];
    	}
    	else 
		if(code >= 512)
		{
    		if(video->intravlc)     
		  	  	tab = &mpeg2_DCTtab1a[(code >> 6) - 8];
    		else              
				tab = &mpeg2_DCTtab1[(code >> 6) - 8];
    	}
    	else 
		if(code >= 256) tab = &mpeg2_DCTtab2[(code >> 4) - 16];
    	else 
		if(code >= 128) tab = &mpeg2_DCTtab3[(code >> 3) - 16];
    	else 
		if(code >= 64)  tab = &mpeg2_DCTtab4[(code >> 2) - 16];
    	else 
		if(code >= 32)  tab = &mpeg2_DCTtab5[(code >> 1) - 16];
    	else 
		if(code >= 16)  tab = &mpeg2_DCTtab6[code - 16];
    	else 
		{
    		fprintf(stderr,"mpeg2video_getmpg2intrablock: invalid Huffman code\n");
    		video->fault = 1;
    		return 1;
    	}

    	mpeg2bits_flushbits(video->vstream, tab->len);

// end_of_block
    	if(tab->run == 64)
    	   	break; 

    	if(tab->run == 65)
		{        
// escape
    	  	i += mpeg2bits_getbits(video->vstream, 6);

    	  	val = mpeg2bits_getbits(video->vstream, 12);
    	  	if((val & 2047) == 0)
			{
        		fprintf(stderr,"mpeg2video_getmpg2intrablock: invalid signed_level (escape)\n");
        		video->fault = 1;
        		return;
    	  	}
    	  	if((sign = (val >= 2048)) != 0) val = 4096 - val;
    	}
    	else 
		{
    		i += tab->run;
    		val = tab->level;
    		sign = mpeg2bits_getbits(video->vstream, 1);
    	}

    	j = (layerdata->altscan ? mpeg2_alternate_scan : mpeg2_zig_zag_scan)[i];

#ifdef HAVE_MMX
    	val = (val * layerdata->quant_scale * qmat[j]);
#else
    	val = (val * layerdata->quant_scale * qmat[j]) >> 4;
#endif

    	bp[j] = sign ? -val : val;
    	nc++;
	}

	if(j != 0) 
	{
/* not a sparse matrix ! */
    	 layerdata->sparse[comp] = 0;
	}
	return 1;
}


// decode one non-intra coded MPEG-2 block

int mpeg2video_getmpg2interblock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp)
{
	register int val, i, j, sign, nc;
	unsigned int code;
	mpeg2_DCTtab_t *tab;
	short *bp;
	int *qmat;

/* with data partitioning, data always goes to base layer */
  	bp = layerdata->block[comp];

  	qmat = (comp < 4 || video->chroma_format == CHROMA420)
         ? layerdata->non_intra_quantizer_matrix
         : layerdata->chroma_non_intra_quantizer_matrix;

  	nc = 0;

/* decode AC coefficients */
  	for(i = 0; ; i++)
	{
    	code = mpeg2bits_showbits(video->vstream, 16);
    	if(code >= 16384)
		{
    	  if(i == 0) tab = &mpeg2_DCTtabfirst[(code >> 12) - 4];
    	  else      tab = &mpeg2_DCTtabnext[(code >> 12) - 4];
    	}
    	else 
		if(code >= 1024) tab = &mpeg2_DCTtab0[(code >> 8) - 4];
    	else 
		if(code >= 512)  tab = &mpeg2_DCTtab1[(code >> 6) - 8];
    	else 
		if(code >= 256)  tab = &mpeg2_DCTtab2[(code >> 4) - 16];
    	else 
		if(code >= 128)  tab = &mpeg2_DCTtab3[(code >> 3) - 16];
    	else 
		if(code >= 64)   tab = &mpeg2_DCTtab4[(code >> 2) - 16];
    	else 
		if(code >= 32)   tab = &mpeg2_DCTtab5[(code >> 1) - 16];
    	else 
		if(code >= 16)   tab = &mpeg2_DCTtab6[code - 16];
    	else 
		{
    		fprintf(stderr, "mpeg2video_getmpg2interblock: invalid Huffman code\n");
    		video->fault = 1;
    		return;
    	}

    	mpeg2bits_flushbits(video->vstream, tab->len);

// end_of_block
    	if(tab->run == 64)
       		break;          

    	if(tab->run == 65)
		{                 
// escape
    		i += mpeg2bits_getbits(video->vstream, 6);
    		val = mpeg2bits_getbits(video->vstream, 12);
    		if((val & 2047) == 0)
			{
        		fprintf(stderr, "mpeg2video_getmpg2interblock: invalid signed_level (escape)\n");
        		video->fault = 1;
        		return 1;
    		}
    		if((sign = (val >= 2048)) != 0) val = 4096 - val;
    	}
    	else 
		{
    		i += tab->run;
    		val = tab->level;
    		sign = mpeg2bits_getbits(video->vstream, 1);
    	}

    	j = (layerdata->altscan ? mpeg2_alternate_scan : mpeg2_zig_zag_scan)[i];
#ifdef HAVE_MMX
     	val = (((val << 1)+1) * layerdata->quant_scale * qmat[j]) >> 1;
#else
     	val = (((val << 1)+1) * layerdata->quant_scale * qmat[j]) >> 5;
#endif

    	bp[j] = sign ? (-val) : val ;
    	nc++;
	}

	if(j != 0) 
	{
      	layerdata->sparse[comp] = 0;
	}
	return 0;
}


/* set block to zero */

int mpeg2video_clearblock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp, int size)
{
	layerdata->sparse[comp] = 1;
	memset(layerdata->block[comp], 0, sizeof(short) * 64 * size);
	return 0;
}

/* limit coefficients to -2048..2047 */

/* move/add 8x8-Block from block[comp] to refframe */

int mpeg2video_addblock(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int comp, int bx, int by, int dct_type, int addflag)
{
	int cc, i, iincr;
	unsigned char *rfp;
	short *bp;
	int spar = layerdata->sparse[comp];

/* color component index */
  	cc = (comp < 4) ? 0 : (comp & 1) + 1; 

  	if(cc == 0)
	{   
/* luminance */
    	if(video->pict_struct == FRAME_PICTURE)
		{
      		if(dct_type)
			{
/* field DCT coding */
        		rfp = video->newframe[0] + 
              		video->coded_picture_width * (by + ((comp & 2) >> 1)) + bx + ((comp & 1) << 3);
        		iincr = (video->coded_picture_width << 1);
      		}
      		else
			{
/* frame DCT coding */
        		rfp = video->newframe[0] + 
             		video->coded_picture_width * (by + ((comp & 2) << 2)) + bx + ((comp & 1) << 3);
        		iincr = video->coded_picture_width;
      		}
		}
    	else 
		{
/* field picture */
      		rfp = video->newframe[0] + 
           		(video->coded_picture_width << 1) * (by + ((comp & 2) << 2)) + bx + ((comp & 1) << 3);
      		iincr = (video->coded_picture_width << 1);
    	}
 	}
  	else 
	{
/* chrominance */

/* scale coordinates */
    	if(video->chroma_format != CHROMA444) bx >>= 1;
    	if(video->chroma_format == CHROMA420) by >>= 1;
    	if(video->pict_struct == FRAME_PICTURE)
		{
    		if(dct_type && (video->chroma_format != CHROMA420))
			{
/* field DCT coding */
        		rfp = video->newframe[cc]
            		  + video->chrom_width * (by + ((comp & 2) >> 1)) + bx + (comp & 8);
        		iincr = (video->chrom_width << 1);
    		}
    		else 
			{
/* frame DCT coding */
        		rfp = video->newframe[cc]
            		  + video->chrom_width * (by + ((comp & 2) << 2)) + bx + (comp & 8);
        		iincr = video->chrom_width;
    		}
    	}
    	else 
		{
/* field picture */
    		rfp = video->newframe[cc]
            	  + (video->chrom_width << 1) * (by + ((comp & 2) << 2)) + bx + (comp & 8);
    		iincr = (video->chrom_width << 1);
    	}
  	}

  	bp = layerdata->block[comp];

  if(addflag) 
  {
#ifdef HAVE_MMX
      if(spar) 
	  {
       __asm__ __volatile__(
            "movq       (%2),%%mm6\n"  /* 4 blockvals */
            "pxor       %%mm4,%%mm4\n"
            "punpcklwd  %%mm6,%%mm6\n"
            "punpcklwd  %%mm6,%%mm6\n"
            ".align 8\n"
            "1:"
                "movq       (%1),  %%mm0\n"     /* 8 rindex1 */
                "movq       %%mm0, %%mm2\n"
                "punpcklbw  %%mm4, %%mm0\n"
                "punpckhbw  %%mm4, %%mm2\n"
                "paddw      %%mm6, %%mm0\n"
                "paddw      %%mm6, %%mm2\n"

                "packuswb   %%mm2, %%mm0\n"
                "movq       %%mm0, (%1)\n"

                "leal       (%1,%3), %1\n"
              "loop       1b\n"
              :              /* scr   dest */
              : "c" (8),"r" (rfp), "r" (bp), "r" (iincr)
                );
    	}
    	else 
		{
       __asm__ __volatile__(
         "pxor    %%mm4,%%mm4\n"

         ".align 8\n"
         "1:"
           "movq       (%2), %%mm0\n"   /* 8 rfp 0 1 2 3 4 5 6 7*/
           "movq       (%1), %%mm6\n"   /* 4 blockvals 0 1 2 3 */

           "movq       %%mm0, %%mm2\n"
           "movq       8(%1), %%mm5\n"  /* 4 blockvals 0 1 2 3 */
           "punpcklbw  %%mm4, %%mm0\n"  /* 0 2 4 6 */
           "punpckhbw  %%mm4, %%mm2\n"  /* 1 3 5 7 */

           "paddw      %%mm6, %%mm0\n"
           "paddw      %%mm5, %%mm2\n"
           "packuswb   %%mm2, %%mm0\n"

           "addl       $16, %1\n"
           "movq       %%mm0, (%2)\n"

           "leal       (%2,%3), %2\n"
         "loop       1b\n"
         :              /* scr   dest */
         : "c" (8),"r" (bp), "r" (rfp), "r" (iincr)
       );

//      add_block_mmx(rfp,bp,iincr);
    	}
#else
		for(i = 0; i < 8; i++)
		{
    		rfp[0] = CLIP(bp[0] + rfp[0]);
    		rfp[1] = CLIP(bp[1] + rfp[1]);
    		rfp[2] = CLIP(bp[2] + rfp[2]);
    		rfp[3] = CLIP(bp[3] + rfp[3]);
    		rfp[4] = CLIP(bp[4] + rfp[4]);
    		rfp[5] = CLIP(bp[5] + rfp[5]);
    		rfp[6] = CLIP(bp[6] + rfp[6]);
    		rfp[7] = CLIP(bp[7] + rfp[7]);
    		rfp += iincr;
    		bp += 8;
		}
#endif
  	}
  	else 
  	{
#ifdef HAVE_MMX
    	if(spar) 
		{
            __asm__ __volatile__(
            "movd       (%2),           %%mm0\n"                // " 0 0 0  v1"
            "punpcklwd  %%mm0,          %%mm0\n"    // " 0 0 v1 v1"
            "punpcklwd  %%mm0,          %%mm0\n"
            "paddw      MMX_128,        %%mm0\n"
            "packuswb   %%mm0,          %%mm0\n"
            "leal       (%0,%1,2),      %%eax\n"

            "movq        %%mm0,         (%0, %1)\n"
            "movq        %%mm0,         (%%eax)\n"
            "leal        (%%eax,%1,2),  %0\n"
            "movq        %%mm0,         (%%eax, %1)\n"

            "movq        %%mm0,         (%0)\n"
            "leal        (%0,%1,2),     %%eax\n"
            "movq        %%mm0,         (%0, %1)\n"

            "movq        %%mm0,         (%%eax)\n"
            "movq        %%mm0,         (%%eax, %1)\n"
            :
            : "D" (rfp), "c" (iincr), "b" (bp)
            : "eax");
    	}
    	else 
		{

       __asm__ __volatile__(
            "movq        MMX_128,%%mm4\n"
            ".align 8\n"
            "1:"
              "movq      (%1),   %%mm0\n"
              "movq      8(%1),  %%mm1\n"
              "paddw     %%mm4,  %%mm0\n"

              "movq      16(%1), %%mm2\n"
              "paddw     %%mm4,  %%mm1\n"

              "movq      24(%1), %%mm3\n"
              "paddw     %%mm4,  %%mm2\n"

              "packuswb  %%mm1,  %%mm0\n"
              "paddw     %%mm4,  %%mm3\n"

              "addl $32, %1\n"
              "packuswb  %%mm3,  %%mm2\n"

              "movq   %%mm0, (%2)\n"

              "movq   %%mm2, (%2,%3)\n"

              "leal       (%2,%3,2), %2\n"
            "loop       1b\n"
            :
            : "c" (4), "r" (bp), "r" (rfp), "r" (iincr)
        );

//      set_block_mmx(rfp,bp,iincr);
    	}
#else
    	for(i = 0; i < 8; i++)
		{
    		rfp[0] = CLIP(bp[0] + 128);
    		rfp[1] = CLIP(bp[1] + 128);
    		rfp[2] = CLIP(bp[2] + 128);
    		rfp[3] = CLIP(bp[3] + 128);
    		rfp[4] = CLIP(bp[4] + 128);
    		rfp[5] = CLIP(bp[5] + 128);
    		rfp[6] = CLIP(bp[6] + 128);
    		rfp[7] = CLIP(bp[7] + 128);
    		rfp+= iincr;
    		bp += 8;
    	}
#endif
  	}
}

/* decode all macroblocks of the current picture */

int mpeg2video_get_macroblocks(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, int framenum)
{
	int comp;
	int MBA, MBAmax, MBAinc, mb_type, cbp, motion_type = 0, dct_type;
	int slice_vert_pos_ext;
	int bx, by;
	unsigned int code;
	int dc_dct_pred[3];
	int mv_count, mv_format, mvscale;
	int PMV[2][2][2], mv_field_sel[2][2];
	int dmv, dmvector[2];
	int qs;
	int stwtype, stwclass; 
	int SNRcbp;


/* number of macroblocks per picture */
  	MBAmax = video->mb_width * video->mb_height;

/* field picture has half as mnay macroblocks as frame */
	if(video->pict_struct != FRAME_PICTURE)
	    MBAmax >>= 1; 

/* macroblock address */
  	MBA = 0; 
  	MBAinc = 0;

  	video->fault = 0;

  	for(;;)
	{
    	if(!video->prog_seq && 
			video->pict_struct == FRAME_PICTURE && 
			MBA == (MBAmax >> 1) &&
        	framenum > -1)
		{
// Called once for every dither
			mpeg2video_display_second_field(video);
		}

	    if(MBAinc == 0)
		{
/* startcode or fault */
	 		if(!mpeg2bits_showbits(video->vstream, 23) || video->fault)
			{ 
/* if fault: resynchronize to next startcode */
resync: 
		        video->fault = 0;
/* all macroblocks decoded */
// Exit here.
        		if(MBA >= MBAmax) return;
				
        		code = mpeg2bits_startcode(video->vstream);
				if(mpeg2bits_error(video->vstream)) return 1;

        		if(code < MPEG2_SLICE_MIN_START || code > MPEG2_SLICE_MAX_START)
				{
/* only slice headers are allowed in picture_data */
          			fprintf(stderr, "mpeg2video_get_macroblocks: premature end of picture.\n");
          			return 1;
       			}
        		mpeg2bits_flushbits(video->vstream, 32);

/* decode slice header (may change quant_scale) */
        		slice_vert_pos_ext = mpeg2video_getslicehdr(video, layerdata);

/* decode macroblock address increment */
        		MBAinc = mpeg2video_get_macroblock_address(video, layerdata);

				if(video->fault) goto resync;

/* set current location */
        		MBA = ((slice_vert_pos_ext << 7) + (code & 255) - 1) * video->mb_width + MBAinc - 1;
/* first macroblock in slice: not skipped */
        		MBAinc = 1; 

/* reset all DC coefficient and motion vector predictors */
        		dc_dct_pred[0] = dc_dct_pred[1] = dc_dct_pred[2] = 0;
        		PMV[0][0][0] = PMV[0][0][1] = PMV[1][0][0] = PMV[1][0][1] = 0;
        		PMV[0][1][0] = PMV[0][1][1] = PMV[1][1][0] = PMV[1][1][1] = 0;
      		}
      		else 
			{ 
/* neither startcode nor fault */
        		if(MBA >= MBAmax)
				{
          			fprintf(stderr, "mpeg2video_get_macroblocks: too many macroblocks in picture\n");
          			return 1;
        		}

/* decode macroblock address increment */
        		MBAinc = mpeg2video_get_macroblock_address(video, layerdata);

        		if(video->fault) goto resync;
      		}
    	}

    	if(MBA >= MBAmax)
		{
/* MBAinc points beyond picture dimensions */
      		fprintf(stderr, "mpeg2video_get_macroblocks: too many macroblocks in picture\n");
      		return 1;
    	}

/* not skipped */
    	if(MBAinc == 1) 
		{
			mpeg2video_macroblock_modes(video, layerdata, &mb_type, &stwtype, &stwclass,
        		&motion_type, &mv_count, &mv_format, &dmv, &mvscale, &dct_type);

			if(video->fault) goto resync;

      		if(mb_type & MB_QUANT)
			{
        		qs = mpeg2bits_getbits(video->vstream, 5);

        		if(layerdata->mpeg2)
            	 	layerdata->quant_scale = layerdata->qscale_type ? mpeg2_non_linear_mquant_table[qs] : (qs << 1);
        		else 
					layerdata->quant_scale = qs;

        		if(layerdata->scalable_mode == SC_DP)
/* make sure quant_scale is valid */
          			layerdata->quant_scale = layerdata->quant_scale;
      		}

/* motion vectors */

/* decode forward motion vectors */
      		if((mb_type & MB_FORWARD) || ((mb_type & MB_INTRA) && video->conceal_mv))
			{
        		if(layerdata->mpeg2)
        			mpeg2video_motion_vectors(video, layerdata, PMV, dmvector, mv_field_sel, 
            			0, mv_count, mv_format, video->h_forw_r_size, video->v_forw_r_size, 
						dmv, mvscale);
        		else
        		  	mpeg2video_motion_vector(video, layerdata, PMV[0][0], dmvector, 
            			video->forw_r_size, video->forw_r_size, 0, 0, video->full_forw);
    		}
      		if(video->fault) return 1;

/* decode backward motion vectors */
    		if(mb_type & MB_BACKWARD)
			{
        		if(layerdata->mpeg2)
        		  	mpeg2video_motion_vectors(video, layerdata, PMV, dmvector, mv_field_sel, 
            			1, mv_count, mv_format, video->h_back_r_size, video->v_back_r_size, 
						0, mvscale);
        		else
        		  	mpeg2video_motion_vector(video, layerdata, PMV[0][1], dmvector, 
            			video->back_r_size, video->back_r_size, 0, 0, video->full_back);
    		}

      		if(video->fault) return 1;

/* remove marker_bit */
      		if((mb_type & MB_INTRA) && video->conceal_mv)
        		mpeg2bits_flushbits(video->vstream, 1); 

/* macroblock_pattern */
      		if(mb_type & MB_PATTERN)
			{
        		cbp = mpeg2video_get_cbp(video);
        		if(video->chroma_format == CHROMA422)
				{
/* coded_block_pattern_1 */
        		  	cbp = (cbp << 2) | mpeg2bits_getbits(video->vstream, 2); 
        		}
        		else 
				if(video->chroma_format == CHROMA444)
				{
/* coded_block_pattern_2 */
        		  	cbp = (cbp << 6) | mpeg2bits_getbits(video->vstream, 6); 
        		}
    		}
    		else
        	  	cbp = (mb_type & MB_INTRA) ? ((1 << video->blk_cnt) - 1) : 0;

      		if(video->fault) return 1;

/* decode blocks */
      		mpeg2video_clearblock(video, layerdata, 0, video->blk_cnt);
      		for(comp = 0; comp < video->blk_cnt; comp++)
			{
        		if(cbp & (1 << (video->blk_cnt - comp - 1)))
				{
          			if(mb_type & MB_INTRA)
					{
            			if(layerdata->mpeg2) 
							mpeg2video_getmpg2intrablock(video, layerdata, comp, dc_dct_pred);
            			else           
							mpeg2video_getintrablock(video, layerdata, comp, dc_dct_pred);
          			}
        			else 
					{
            		  	if(layerdata->mpeg2) 
					  		mpeg2video_getmpg2interblock(video, layerdata, comp);
            		  	else           
					  		mpeg2video_getinterblock(video, layerdata, comp);
        			}
        			if(video->fault) return 1;
        		}
      		}

/* reset intra_dc predictors */
			if(!(mb_type & MB_INTRA))
        	  	dc_dct_pred[0] = dc_dct_pred[1] = dc_dct_pred[2] = 0;

/* reset motion vector predictors */
    		if((mb_type & MB_INTRA) && !video->conceal_mv)
			{
/* intra mb without concealment motion vectors */
        		PMV[0][0][0] = PMV[0][0][1] = PMV[1][0][0] = PMV[1][0][1] = 0;
        		PMV[0][1][0] = PMV[0][1][1] = PMV[1][1][0] = PMV[1][1][1] = 0;
    		}

    		if((video->pict_type == P_TYPE) && !(mb_type & (MB_FORWARD | MB_INTRA)))
			{
/* non-intra mb without forward mv in a P picture */
        		PMV[0][0][0] = PMV[0][0][1] = PMV[1][0][0] = PMV[1][0][1] = 0;

/* derive motion_type */
        		if(video->pict_struct == FRAME_PICTURE) motion_type = MC_FRAME;
        		else
        		{
        			motion_type = MC_FIELD;
/* predict from field of same parity */
        			mv_field_sel[0][0] = (video->pict_struct == BOTTOM_FIELD);
        		}
      		}

    		if(stwclass == 4)
    		{
/* purely spatially predicted macroblock */
        		PMV[0][0][0] = PMV[0][0][1] = PMV[1][0][0] = PMV[1][0][1] = 0;
        		PMV[0][1][0] = PMV[0][1][1] = PMV[1][1][0] = PMV[1][1][1] = 0;
    		}
    	}
    	else 
		{ 
/* MBAinc!=1: skipped macroblock */
      		mpeg2video_clearblock(video, layerdata, 0, video->blk_cnt);

/* reset intra_dc predictors */
      		dc_dct_pred[0] = dc_dct_pred[1] = dc_dct_pred[2] = 0;

/* reset motion vector predictors */
      		if(video->pict_type == P_TYPE)
        		PMV[0][0][0] = PMV[0][0][1] = PMV[1][0][0] = PMV[1][0][1] = 0;

/* derive motion_type */
      		if(video->pict_struct == FRAME_PICTURE)
        		motion_type = MC_FRAME;
    		else
    		{
        		motion_type = MC_FIELD;
/* predict from field of same parity */
        		mv_field_sel[0][0] = mv_field_sel[0][1] = (video->pict_struct == BOTTOM_FIELD);
    		}

/* skipped I are spatial-only predicted, */
/* skipped P and B are temporal-only predicted */
      		stwtype = (video->pict_type == I_TYPE) ? 8 : 0;

/* clear MB_INTRA */
      		mb_type &= ~MB_INTRA;

/* no block data */
      		cbp = 0; 
    	}

    	SNRcbp = 0;

/* pixel coordinates of top left corner of current macroblock */
    	bx = 16 * (MBA % video->mb_width);
    	by = 16 * (MBA / video->mb_width);

/* motion compensation */
    	if(!(mb_type & MB_INTRA))
    	  	mpeg2video_reconstruct(video, layerdata, bx, by, mb_type, motion_type, PMV, mv_field_sel, dmvector, stwtype);

/* copy or add block data into picture */
    	for(comp = 0; comp < video->blk_cnt; comp++)
		{
      		if((cbp | SNRcbp) & (1 << (video->blk_cnt - 1 - comp)))
			{
#ifdef HAVE_MMX
  	  			IDCT_mmx(layerdata->block[comp]);
#else
          		mpeg2video_idct_conversion(layerdata->block[comp]);
#endif
        		mpeg2video_addblock(video, layerdata, comp, bx, by, dct_type, (mb_type & MB_INTRA) == 0);
      		}
    	}

/* advance to next macroblock */
    	MBA++;
    	MBAinc--;
  	}
}

// decode one frame or field picture

int mpeg2video_getpicture(mpeg2video_t *video, int framenum)
{
	int i, result = 0;
	if(video->pict_struct == FRAME_PICTURE && video->secondfield)
	{
/* recover from illegal number of field pictures */
    	fprintf(stderr, "mpeg2video_getpicture: odd number of field pictures\n");
    	video->secondfield = 0;
	}

  	for(i = 0; i < 3; i++)
	{
    	if(video->pict_type == B_TYPE) 
			video->newframe[i] = video->auxframe[i];
    	else 
		{
    	  	if(!video->secondfield)
			{
        		unsigned char* tmp = video->oldrefframe[i];
        		video->oldrefframe[i] = video->refframe[i];
        		video->refframe[i] = tmp;
    	  	}

    	 	video->newframe[i] = video->refframe[i];
    	}

    	if(video->pict_struct == BOTTOM_FIELD)
    	  video->newframe[i] += (i == 0) ? video->coded_picture_width : video->chrom_width;
	}

  	result |= mpeg2video_get_macroblocks(video, video->layerdata, framenum);
	if(framenum > -1 && !result)
	{
    	if(video->pict_struct == FRAME_PICTURE || video->secondfield)
		{
     	  	if(video->pict_type == B_TYPE) 
 				mpeg2video_dither(video, video->auxframe);
     	  	else                   
 				mpeg2video_dither(video, video->oldrefframe);
    	}
    	else 
		{
// Called once for every dither
			mpeg2video_display_second_field(video);
		}
	}

  	if(video->pict_struct != FRAME_PICTURE) video->secondfield = !video->secondfield;
	return result;
}
