#include "mpeg2video.h"
#include "vlc.h"

#include <stdio.h>

int mpeg2video_get_macroblock_address(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
	int code, val = 0;

  	while((code = mpeg2bits_showbits(video->vstream, 11)) < 24)
	{
/* Is not macroblock_stuffing */
    	if(code != 15)
		{     
/* Is macroblock_escape */
      		if(code == 8)
			{    
        		val += 33;
      		}
      		else 
			{
        		fprintf(stderr, "mpeg2video_get_macroblock_address: invalid macroblock_address_increment code\n");
        		video->fault = 1;
        		return 1;
      		}
    	}

    	mpeg2bits_flushbits(video->vstream, 11);
  	}

  	if(code >= 1024)
	{
    	mpeg2bits_flushbits(video->vstream, 1);
    	return val + 1;
  	}

  	if(code >= 128)
	{
    	code >>= 6;
    	mpeg2bits_flushbits(video->vstream, mpeg2_MBAtab1[code].len);
    	return val + mpeg2_MBAtab1[code].val;
  	}

  	code -= 24;
  	mpeg2bits_flushbits(video->vstream, mpeg2_MBAtab2[code].len);

  	return val + mpeg2_MBAtab2[code].val;
}

/* macroblock_type for pictures with spatial scalability */

int mpeg2video_getsp_imb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	int code = mpeg2bits_showbits(video->vstream, 4);
	if(code == 0)
	{
    	fprintf(stderr,"mpeg2video_getsp_imb_type: invalid macroblock_type code\n");
    	video->fault = 1;
    	return 0;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_spIMBtab[code].len);
  	return mpeg2_spIMBtab[code].val;
}

int mpeg2video_getsp_pmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	int code = mpeg2bits_showbits(video->vstream, 7);
	if(code < 2)
	{
    	fprintf(stderr,"mpeg2video_getsp_pmb_type: invalid macroblock_type code\n");
    	video->fault = 1;
    	return 0;
  	}

  	if(code >= 16)
	{
    	code >>= 3;
    	mpeg2bits_flushbits(video->vstream, mpeg2_spPMBtab0[code].len);

    	return mpeg2_spPMBtab0[code].val;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_spPMBtab1[code].len);
  	return mpeg2_spPMBtab1[code].val;
}

int mpeg2video_getsp_bmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	mpeg2_VLCtab_t *p;
  	int code = mpeg2bits_showbits(video->vstream, 9);

  	if(code >= 64) 
		p = &mpeg2_spBMBtab0[(code >> 5) - 2];
  	else 
	if(code >= 16) 
		p = &mpeg2_spBMBtab1[(code >> 2) - 4];
  	else 
	if(code >= 8)  
		p = &mpeg2_spBMBtab2[code - 8];
  	else 
	{
    	fprintf(stderr,"mpeg2video_getsp_bmb_type: invalid macroblock_type code\n");
    	video->fault = 1;
    	return 0;
  	}

  	mpeg2bits_flushbits(video->vstream, p->len);
  	return p->val;
}

int mpeg2video_get_imb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
	if(mpeg2bits_getbits(video->vstream, 1))
	{
    	return 1;
  	}

  	if(!mpeg2bits_getbits(video->vstream, 1))
	{
    	fprintf(stderr,"mpeg2video_get_imb_type: invalid macroblock_type code\n");
    	video->fault = 1;
  	}

  	return 17;
}

int mpeg2video_get_pmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	int code;

  	if((code = mpeg2bits_showbits(video->vstream, 6)) >= 8)
	{
    	code >>= 3;
    	mpeg2bits_flushbits(video->vstream, mpeg2_PMBtab0[code].len);
    	return mpeg2_PMBtab0[code].val;
  	}

  	if(code == 0)
	{
    	fprintf(stderr,"mpeg2video_get_pmb_type: invalid macroblock_type code\n");
    	video->fault = 1;
    	return 0;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_PMBtab1[code].len);
  	return mpeg2_PMBtab1[code].val;
}

int mpeg2video_get_bmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	int code;

  	if((code = mpeg2bits_showbits(video->vstream, 6)) >= 8)
	{
    	code >>= 2;
    	mpeg2bits_flushbits(video->vstream, mpeg2_BMBtab0[code].len);
    	return mpeg2_BMBtab0[code].val;
  	}

  	if(code == 0)
	{
    	fprintf(stderr,"mpeg2video_get_bmb_type: invalid macroblock_type code\n");
    	video->fault = 1;
    	return 0;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_BMBtab1[code].len);

	return mpeg2_BMBtab1[code].val;
}

int mpeg2video_get_dmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  	if(!mpeg2bits_getbits(video->vstream, 1))
	{
    	fprintf(stderr,"mpeg2video_get_dmb_type: invalid macroblock_type code\n");
    	video->fault=1;
  	}

  	return 1;
}


int mpeg2video_get_snrmb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
  int code = mpeg2bits_showbits(video->vstream, 3);

  if(code == 0)
  {
      fprintf(stderr,"mpeg2video_get_snrmb_type: invalid macroblock_type code\n");
      video->fault = 1;
      return 0;
  }

  mpeg2bits_flushbits(video->vstream, mpeg2_SNRMBtab[code].len);
  return mpeg2_SNRMBtab[code].val;
}

int mpeg2video_get_mb_type(mpeg2video_t *video, mpeg2_layerdata_t *layerdata)
{
	if (layerdata->scalable_mode == SC_SNR) return mpeg2video_get_snrmb_type(video, layerdata);
	else
	{
    	switch(video->pict_type)
		{
    		case I_TYPE: return layerdata->pict_scal ? mpeg2video_getsp_imb_type(video, layerdata) : mpeg2video_get_imb_type(video, layerdata);
    		case P_TYPE: return layerdata->pict_scal ? mpeg2video_getsp_pmb_type(video, layerdata) : mpeg2video_get_pmb_type(video, layerdata);
    		case B_TYPE: return layerdata->pict_scal ? mpeg2video_getsp_bmb_type(video, layerdata) : mpeg2video_get_bmb_type(video, layerdata);
    		case D_TYPE: return mpeg2video_get_dmb_type(video, layerdata);
    		default: 
				fprintf(stderr, "mpeg2video_getmbtype: unknown coding type\n");
/* MPEG-1 only, not implemented */
	  	}
  	}

  	return 0;
}

int mpeg2video_macroblock_modes(mpeg2video_t *video, mpeg2_layerdata_t *layerdata, 
	int *pmb_type, int *pstwtype, int *pstwclass, int *pmotion_type, 
	int *pmv_count, int *pmv_format, int *pdmv, int *pmvscale,
	int *pdct_type)
{
	int mb_type;
	int stwtype, stwcode, stwclass;
	int motion_type = 0, mv_count, mv_format, dmv, mvscale;
	int dct_type;
	static unsigned char stwc_table[3][4]
    	= { {6,3,7,4}, {2,1,5,4}, {2,5,7,4} };
	static unsigned char stwclass_table[9]
    	= {0, 1, 2, 1, 1, 2, 3, 3, 4};

/* get macroblock_type */
  	mb_type = mpeg2video_get_mb_type(video, layerdata);

  	if(video->fault) return;

/* get spatial_temporal_weight_code */
  	if(mb_type & MB_WEIGHT)
  	{
    	if(video->stwc_table_index == 0)
      		stwtype = 4;
    		else
    	{
      		stwcode = mpeg2bits_getbits(video->vstream, 2);
      		stwtype = stwc_table[video->stwc_table_index - 1][stwcode];
    	}
  	}
  	else
    	stwtype = (mb_type & MB_CLASS4) ? 8 : 0;

/* derive spatial_temporal_weight_class (Table 7-18) */
  	stwclass = stwclass_table[stwtype];

/* get frame/field motion type */
  	if(mb_type & (MB_FORWARD | MB_BACKWARD))
	{
    	if(video->pict_struct == FRAME_PICTURE)
		{ 
/* frame_motion_type */
      		motion_type = video->frame_pred_dct ? MC_FRAME : mpeg2bits_getbits(video->vstream, 2);
    	}
    	else 
		{ 
/* field_motion_type */
      		motion_type = mpeg2bits_getbits(video->vstream, 2);
    	}
  	}
  	else 
	if((mb_type & MB_INTRA) && video->conceal_mv)
  	{
/* concealment motion vectors */
    	motion_type = (video->pict_struct == FRAME_PICTURE) ? MC_FRAME : MC_FIELD;
  	}

/* derive mv_count, mv_format and dmv, (table 6-17, 6-18) */
  	if(video->pict_struct == FRAME_PICTURE)
  	{
    	mv_count = (motion_type == MC_FIELD && stwclass < 2) ? 2 : 1;
    	mv_format = (motion_type == MC_FRAME) ? MV_FRAME : MV_FIELD;
  	}
  	else
  	{
    	mv_count = (motion_type == MC_16X8) ? 2 : 1;
    	mv_format = MV_FIELD;
  	}

  	dmv = (motion_type == MC_DMV); /* dual prime */

/* field mv predictions in frame pictures have to be scaled */
  	mvscale = ((mv_format == MV_FIELD) && (video->pict_struct == FRAME_PICTURE));

/* get dct_type (frame DCT / field DCT) */
  	dct_type = (video->pict_struct == FRAME_PICTURE) && 
             	(!video->frame_pred_dct) && 
             	(mb_type & (MB_PATTERN | MB_INTRA)) ? 
             	mpeg2bits_getbits(video->vstream, 1) : 0;

/* return values */
	*pmb_type = mb_type;
	*pstwtype = stwtype;
	*pstwclass = stwclass;
	*pmotion_type = motion_type;
	*pmv_count = mv_count;
	*pmv_format = mv_format;
	*pdmv = dmv;
	*pmvscale = mvscale;
	*pdct_type = dct_type;
	return 0;
}
