#include "mpeg2video.h"
#include "vlc.h"

#include <stdio.h>


/* calculate motion vector component */

int mpeg2video_calc_mv(int *pred, int r_size, int motion_code, int motion_r, int full_pel_vector)
{
	int lim = 16 << r_size;
	int vec = full_pel_vector ? (*pred >> 1) : (*pred);

	if(motion_code > 0)
	{
    	vec += ((motion_code - 1) << r_size) + motion_r + 1;
    	if(vec >= lim) vec -= lim + lim;
	}
	else 
	if(motion_code < 0)
	{
    	vec -= ((-motion_code - 1) << r_size) + motion_r + 1;
    	if(vec < -lim) vec += lim + lim;
	}
	*pred = full_pel_vector ? (vec << 1) : vec;
	return 0;
}


int mpeg2video_calc_dmv(mpeg2video_t *video, int DMV[][2], int *dmvector, int mvx, int mvy)
{
/*
int *dmvector, * differential motion vector *
int mvx, int mvy  * decoded mv components (always in field format) *
*/
	if(video->pict_struct == FRAME_PICTURE)
	{
    	if(video->topfirst)
		{
/* vector for prediction of top field from bottom field */
    		DMV[0][0] = ((mvx  +(mvx>0))>>1) + dmvector[0];
    		DMV[0][1] = ((mvy  +(mvy>0))>>1) + dmvector[1] - 1;

/* vector for prediction of bottom field from top field */
    		DMV[1][0] = ((3*mvx+(mvx>0))>>1) + dmvector[0];
    		DMV[1][1] = ((3*mvy+(mvy>0))>>1) + dmvector[1] + 1;
    	}
    	else 
		{
/* vector for prediction of top field from bottom field */
    		DMV[0][0] = ((3*mvx+(mvx>0))>>1) + dmvector[0];
    		DMV[0][1] = ((3*mvy+(mvy>0))>>1) + dmvector[1] - 1;

/* vector for prediction of bottom field from top field */
    		DMV[1][0] = ((mvx  +(mvx>0))>>1) + dmvector[0];
    		DMV[1][1] = ((mvy  +(mvy>0))>>1) + dmvector[1] + 1;
    	}
	}
	else 
	{
/* vector for prediction from field of opposite 'parity' */
    	DMV[0][0] = ((mvx+(mvx>0))>>1) + dmvector[0];
    	DMV[0][1] = ((mvy+(mvy>0))>>1) + dmvector[1];

/* correct for vertical field shift */
    	if (video->pict_struct == TOP_FIELD) DMV[0][1]--;
    	else                        DMV[0][1]++;
	}
}

int mpeg2video_get_mv(mpeg2video_t *video)
{
  	int code;

  	if(mpeg2bits_getbits(video->vstream, 1))
	{
    	return 0;
  	}

  	if((code = mpeg2bits_showbits(video->vstream, 9)) >= 64)
	{
    	code >>= 6;
    	mpeg2bits_flushbits(video->vstream, mpeg2_MVtab0[code].len);
    	return mpeg2bits_getbits(video->vstream, 1) ? -mpeg2_MVtab0[code].val : mpeg2_MVtab0[code].val;
  	}

  	if(code >= 24)
	{
    	code >>= 3;
    	mpeg2bits_flushbits(video->vstream, mpeg2_MVtab1[code].len);
	    return mpeg2bits_getbits(video->vstream, 1) ? -mpeg2_MVtab1[code].val : mpeg2_MVtab1[code].val;
  	}

  	if((code -= 12) < 0)
	{
    	fprintf(stderr,"mpeg2video_get_mv: invalid motion_vector code\n");
    	video->fault = 1;
    	return 1;
  	}

  	mpeg2bits_flushbits(video->vstream, mpeg2_MVtab2[code].len);
 	return mpeg2bits_getbits(video->vstream, 1) ? -mpeg2_MVtab2[code].val : mpeg2_MVtab2[code].val;
}

/* get differential motion vector (for dual prime prediction) */

int mpeg2video_get_dmv(mpeg2video_t *video)
{
  	if(mpeg2bits_getbits(video->vstream, 1))
	{
    	return mpeg2bits_getbits(video->vstream, 1) ? -1 : 1;
  	}
  	else 
	{
    	return 0;
  	}
}



/* get and decode motion vector and differential motion vector */

int mpeg2video_motion_vector(mpeg2video_t *video, mpeg2_layerdata_t *layerdata,
	int *PMV, int *dmvector, int h_r_size, int v_r_size,
	int dmv, int mvscale, int full_pel_vector)
{
	int motion_r;
	int motion_code = mpeg2video_get_mv(video);

	if(video->fault) return 1;
	motion_r = (h_r_size != 0 && motion_code != 0) ? mpeg2bits_getbits(video->vstream, h_r_size) : 0;

  	mpeg2video_calc_mv(&PMV[0], h_r_size, motion_code, motion_r, full_pel_vector);

  	if(dmv) dmvector[0] = mpeg2video_get_dmv(video);

  	motion_code = mpeg2video_get_mv(video);
  	if(video->fault)  return 1;
  	motion_r = (v_r_size != 0 && motion_code != 0) ? mpeg2bits_getbits(video->vstream, v_r_size) : 0;

/* DIV 2 */
  	if(mvscale) PMV[1] >>= 1; 

  	mpeg2video_calc_mv(&PMV[1], v_r_size, motion_code, motion_r, full_pel_vector);

	if(mvscale) PMV[1] <<= 1;
	if(dmv) dmvector[1] = mpeg2video_get_dmv(video);
	return video->fault;
}

int mpeg2video_motion_vectors(mpeg2video_t *video, mpeg2_layerdata_t *layerdata,
	int PMV[2][2][2], int dmvector[2], int mv_field_sel[2][2],
	int s, int mv_count, int mv_format, int h_r_size, int v_r_size, 
	int dmv, int mvscale)
{
	int result = 0;
	if(mv_count == 1)
	{
		if(mv_format == MV_FIELD && !dmv)
		{
			mv_field_sel[1][s] = mv_field_sel[0][s] = mpeg2bits_getbits(video->vstream, 1);
		}

    	mpeg2video_motion_vector(video, layerdata, PMV[0][s], dmvector, h_r_size, v_r_size, dmv, mvscale, 0);
    	if(video->fault) return 1;

/* update other motion vector predictors */
    	PMV[1][s][0] = PMV[0][s][0];
    	PMV[1][s][1] = PMV[0][s][1];
  	}
  	else 
	{
    	mv_field_sel[0][s] = mpeg2bits_getbits(video->vstream, 1);
    	mpeg2video_motion_vector(video, layerdata, PMV[0][s], dmvector, h_r_size, v_r_size, dmv, mvscale, 0);
    	if(video->fault) return 1;

    	mv_field_sel[1][s] = mpeg2bits_getbits(video->vstream, 1);
    	mpeg2video_motion_vector(video, layerdata, PMV[1][s], dmvector, h_r_size, v_r_size, dmv, mvscale, 0);
    	if(video->fault) return 1;
  	}
	return 0;
}
