#ifndef AUDIODEVICE_H
#define AUDIODEVICE_H

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#ifndef OTHERAUDIO
#include <sys/soundcard.h>
#endif
#include <unistd.h>

#include "audiodevice.inc"
#include "binary.h"
#include "dcoffset.inc"
#include "maxchannels.h"
#include "mutex.h"
#include "recordgui.inc"
#include "thread.h"
#include "timer.h"

#ifndef AFMT_S24_LE
#define AFMT_S24_LE AFMT_S16_LE
#endif


class AudioDevice : public Thread
{
public:
	AudioDevice();
	~AudioDevice();

	int open_input(char *idevice, int bits, int rate, int channels, long size);
	int open_output(char *odevice, int bits, int rate, int channels, int size);
	int open_duplex(char *device, int bits, int rate, int channels, long size);
	int close_all();
	int reset_output();
	int restart();
	int set_cloexec_flag(int desc, int value);

// ================================ recording

	int read_buffer(float **input, long samples, int channels, int *over, float *max, long input_offset = 0);  // read from the record device
	int set_record_dither(int bits);             // set dither for the amount of bits in the output file

	int stop_recording();


// ================================== dc offset

	int get_dc_offset(long *output, RecordGUIDCOffsetText **dc_offset_text);             // get and set offset
	int set_dc_offset(long dc_offset, int channel);    // set new offset

	DC_Offset *dc_offset_thread;

// =================================== info

// OSS < 3.9   --> playback before recording
// OSS >= 3.9  --> doesn't matter
	int RECORDBEFOREPLAY;
	Mutex duplex_lock, startup_lock;
	int duplex_init;        // notify playback routines to test the duplex lock
	int rec_dither;   // bits in output file
	int play_dither;  // 1 or 0

	int get_obits();
	int get_ochannels();
	int get_output();
	int get_ichannels();
	int get_ibits();
	int get_input();
	long get_orate();
	long bits_to_fmt(int bits);       // return the OSS flag
	int sizetofrag(int size, int channels, int bits);  // convert the size to the OSS buffer parameter


// ================================== playback

	int arm_buffer(int buffer, float **output, long samples, int channels);
// writes to whichever buffer is free or blocks until one becomes free
	int write_buffer(float **output, long samples, int channels = -1); 
 
// after the last buffer is written call this to terminate
// a seperate buffer for termination is required since the audio device can be
// finished without writing a single byte
	int set_last_buffer();         

	int wait_for_startup();
	int wait_for_completion();      // wait for the playback thread to clean up

	int start_playback();          // start the thread processing buffers
	int interrupt_playback();          // interrupt the playback thread
	int set_play_dither(int status);
// set software positioning on or off
	int set_software_positioning(int status = 1);         
// total samples played
	long current_position();      

	void run();           // play back buffers





	int thread_buffer_num, thread_result;
	int r, w, d;
// paths
	char idevice[1024], odevice[1024], ddevice[1024];
	int obits, ibits, dbits;
	int ochannels, ichannels, dchannels;
	int dsp_in, dsp_out, dsp_duplex;
	long orate, irate, drate;

	long buffer_size[TOTAL_BUFFERS];
	int last_buffer[TOTAL_BUFFERS];    // not written to device
// formatted buffers
	char *buffer[TOTAL_BUFFERS], *input_buffer;
	Mutex play_mutex[TOTAL_BUFFERS], arm_mutex[TOTAL_BUFFERS];
	Mutex timer_lock;
	int arm_buffer_num;

// for position information
	long total_samples, last_buffer_size, position_correction;
	long last_position;  // prevent the counter from going backwards
	long device_buffer;
	Timer buffer_timer, global_timer;
	int is_playing_back, is_recording, global_timer_started, software_position_info;
	int interrupt;
};



#endif
