#include "assets.h"
#include "file.h"
#include "fileplugin.h"
#include "pluginserver.h"
#include "vframe.h"



FilePlugin::FilePlugin(ArrayList<PluginServer*> *plugindb, 
	Asset *asset, 
	File *file)
 : FileBase(asset, file)
{
	reset_parameters();
	this->plugindb = plugindb;
}

FilePlugin::~FilePlugin()
{
	close_file();
}

int FilePlugin::reset_parameters_derived()
{
	plugin = 0;
	prev_track = 0;
	data = 0;
}

int FilePlugin::check_header()
{
	int i;
	int result = 0;
	for(i = 0; i < plugindb->total && !result; i++)
	{
		if(plugindb->values[i]->fileio)
		{
			PluginServer plugin(*(plugindb->values[i]));
			plugin.open_plugin();
			result = plugin.check_header(asset->path);
			plugin.close_plugin();
			if(result)
			{
				asset->format = i | 0x8000;
				return 1;
			}
		}
	}
	return 0;
}

int FilePlugin::open_file(int rd, int wr)
{
	int result = 1;
	if(asset->format & 0x8000)
	{
		if((asset->format & 0x7fff) < plugindb->total)
		{
			plugin = new PluginServer(*(plugindb->values[asset->format & 0x7fff]));
			plugin->open_plugin();
			result = plugin->open_file(asset->path, rd, wr);
			if(result)
			{
				plugin->close_plugin();
				delete plugin;
				plugin = 0;
			}
			else
			{
				read_header();
			}
		}
	}
	return result;
}

int FilePlugin::close_file_derived()
{
	if(plugin)
	{
		plugin->close_file();
		plugin->close_plugin();
		delete plugin;
	}
	if(data)
	{
		delete data;
	}
	data = 0;
	plugin = 0;
}

int FilePlugin::read_header()
{
// Read header info
	asset->audio_data = plugin->get_hasaudio();
	asset->video_data = plugin->get_hasvideo();  
	if(asset->audio_data)
	{
		asset->rate = plugin->get_rate(); 	   
		asset->bits = plugin->get_bits(); 	   
		asset->byte_order = plugin->get_byteorder();   
		asset->signed_ = plugin->get_signed();	   
		asset->header = plugin->get_header();	   
		asset->channels = plugin->get_channels();	   
	}
	if(asset->video_data)
	{
		asset->layers = plugin->get_layers();	   
// Don't want a user configured frame rate to get destroyed
		if(!asset->frame_rate)
			asset->frame_rate = plugin->get_file_framerate();   
		asset->width = plugin->get_width();	   
		asset->height = plugin->get_height();	   
		asset->quality = plugin->get_quality();     
		plugin->get_compression(asset->compression); 
	}  
}

long FilePlugin::get_audio_length()
{
	return plugin->get_audio_length();
}

long FilePlugin::get_video_length()
{
	return plugin->get_video_length();
}

int FilePlugin::seek_end()
{
	return plugin->seek_end();
}

int FilePlugin::seek_start()
{
	return plugin->seek_start();
}

long FilePlugin::get_video_position()
{
	return plugin->get_video_position();
}

long FilePlugin::get_audio_position()
{
	return plugin->get_audio_position();
}

int FilePlugin::set_audio_position(long x)
{
	return plugin->set_audio_position(x);
}

int FilePlugin::set_video_position(long x)
{
	return plugin->set_video_position(x);
}

int FilePlugin::set_channel(int channel)
{
	return plugin->set_channel(channel);
}

int FilePlugin::set_layer(int layer)
{
	return plugin->set_layer(layer);
}

int FilePlugin::write_samples(float **buffer, long len)
{
	return 1;
}

int FilePlugin::write_frames(VFrame ***buffers, long len, int use_alpha, int use_float)
{
	return 1;
}

int FilePlugin::read_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset, 
		float in_x1, float in_y1, float in_x2, float in_y2,
		float out_x1, float out_y1, float out_x2, float out_y2, 
		int alpha, int use_alpha, int use_float, int interpolate)
{
	int result = plugin->read_frame(frame, buffer, byte_offset, 
		in_x1, in_y1, in_x2, in_y2,
		out_x1, out_y1, out_x2, out_y2, 
		alpha, use_alpha, use_float, interpolate);
	return result;
}

VFrame* FilePlugin::read_frame(int use_alpha, int use_float)
{
	VFrame *result = plugin->read_frame(use_alpha, use_float);
	return result;
}

int FilePlugin::read_raw_frame_possible()
{
	int result = plugin->read_raw_frame_possible();
	return result;
}

int FilePlugin::read_raw_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset)
{
	int result = plugin->read_raw_frame(frame, buffer, byte_offset);
	return result;
}



int FilePlugin::read_samples(PluginBuffer *buffer, long offset, long len, 
		int feather, 
		long lfeather_len, float lfeather_gain, float lfeather_slope, 
		long rfeather_len, float rfeather_gain, float rfeather_slope)
{
	int result = plugin->read_samples(buffer, offset, len, feather,
		lfeather_len, lfeather_gain, lfeather_slope, 
		rfeather_len, rfeather_gain, rfeather_slope);
	return result;
}

long FilePlugin::get_memory_usage()
{
	return 1;
}
