#include "deleteallindexes.h"
#include "indexprefs.h"
#include "preferences.h"

IndexPrefs::IndexPrefs(PreferencesWindow *pwindow)
 : BC_SubWindow(150, 10, PREFSUBWINDOW_W, PREFSUBWINDOW_H, MEGREY)
{ this->pwindow = pwindow; }

IndexPrefs::~IndexPrefs()
{
	delete ipath;
	delete ipathtext;
	delete isize;
	delete csize;
	delete deleteall;
	delete smp;
	delete icount;
}

int IndexPrefs::create_objects()
{
	char string[1024];
	int x = 10, y = 35;

	add_border(get_resources()->get_bg_shadow1(),
		get_resources()->get_bg_shadow2(),
		get_resources()->get_bg_color(),
		get_resources()->get_bg_light2(),
		get_resources()->get_bg_light1());
	add_tool(new BC_Title(10, 5, "Performance", LARGEFONT, RED));

	add_tool(new BC_Title(x, y, "Index files go here", MEDIUMFONT, BLACK));
	y += 20;
	add_tool(ipathtext = new IndexPathText(x, y, pwindow, pwindow->index_directory));
	add_tool(ipath = new IndexPath(x + 250, y, pwindow, ipathtext, pwindow->index_directory));
	y += 30;
	add_tool(new BC_Title(x, y + 5, "Size of index file", MEDIUMFONT, BLACK));
	sprintf(string, "%ld", pwindow->index_size);
	add_tool(isize = new IndexSize(x + 120, y, pwindow, string));
	y += 30;
	add_tool(new BC_Title(x, y + 5, "Number of index files to keep", MEDIUMFONT, BLACK));
	sprintf(string, "%ld", pwindow->index_count);
	add_tool(icount = new IndexCount(x + 210, y, pwindow, string));
	y += 30;
	add_tool(new BC_Title(x, y + 5, "Cache items", MEDIUMFONT, BLACK));
	sprintf(string, "%ld", pwindow->cache_size);
	add_tool(csize = new CacheSize(x + 100, y, pwindow, string));
	add_tool(deleteall = new DeleteAllIndexes(pwindow));
	y += 30;
	add_tool(new BC_Title(x, y + 5, "Number of CPUs to use"));
	add_tool(smp = new PrefsSMP(x + 175, y, pwindow));
//printf("done IndexPrefs::create_objects()\n");
}






IndexPath::IndexPath(int x, int y, PreferencesWindow *pwindow, BC_TextBox *textbox, char *text)
 : BrowseButton(x, y, textbox, text, "Index Path", "Select the directory for index files", 1)
{ this->pwindow = pwindow; }

IndexPath::~IndexPath() {}

IndexPathText::IndexPathText(int x, int y, PreferencesWindow *pwindow, char *text)
 : BC_TextBox(x, y, 240, text)
{ this->pwindow = pwindow; }

IndexPathText::~IndexPathText() {}

int IndexPathText::handle_event()
{
	strcpy(pwindow->index_directory, get_text());
}




IndexSize::IndexSize(int x, int y, PreferencesWindow *pwindow, char *text)
 : BC_TextBox(x, y, 100, text)
{ this->pwindow = pwindow; }

int IndexSize::handle_event()
{
  static long result;
  
  result = atol(get_text());
  if(result < 64000) result = 64000;
  //if(result < 500000) result = 500000;
  pwindow->index_size = result;
}



IndexCount::IndexCount(int x, int y, PreferencesWindow *pwindow, char *text)
 : BC_TextBox(x, y, 100, text)
{ this->pwindow = pwindow; }

int IndexCount::handle_event()
{
  static long result;
  
  result = atol(get_text());
  if(result < 1) result = 1;
  pwindow->index_count = result;
}



PrefsSMP::PrefsSMP(int x, int y, PreferencesWindow *pwindow)
 : BC_TextBox(x, y, 100, pwindow->smp + 1)
{ this->pwindow = pwindow; }

PrefsSMP::~PrefsSMP()
{
}

int PrefsSMP::handle_event()
{
	pwindow->smp = atol(get_text()) - 1;
	if(pwindow->smp < 0) pwindow->smp = 0;
}


CacheSize::CacheSize(int x, int y, PreferencesWindow *pwindow, char *text)
 : BC_TextBox(x, y, 100, text)
{ this->pwindow = pwindow; }

int CacheSize::handle_event()
{
  static long result;
  
  result = atol(get_text());
  if(result < 1) result = 1;
  //if(result < 500000) result = 500000;
  pwindow->cache_size = result;
}

