#include "messages.h"
#include "pluginbuffer.h"
#include "plugincommands.h"
#include "pluginserver.h"
#include "vframe.h"

// ================================ File I/O

int PluginServer::get_audio_parameters()
{
	if(!plugin_open) return 0;
	messages->write_message(GET_AUDIO_PARAMETERS);
// handle plugin requests while getting parameters
	plugin_server_loop();
	int result = messages->read_message();
	return result;
}

int PluginServer::get_video_parameters()
{
	if(!plugin_open) return 0;
	messages->write_message(GET_VIDEO_PARAMETERS);
// handle plugin requests while getting parameters
	plugin_server_loop();
	int result = messages->read_message();
	return result;
}

int PluginServer::check_header(char *path)
{
	if(!plugin_open) return 0;
	messages->write_message(CHECK_HEADER);
	messages->write_message(path);
	plugin_server_loop();
	int result = messages->read_message();
	return result;
}

int PluginServer::open_file(char *path, int wr, int rd)
{
	if(!plugin_open) return 0;
	messages->write_message(OPEN_FILE);
	messages->write_message(path);
	messages->write_message(rd, wr);
	plugin_server_loop();
	int result = messages->read_message();
	return result;
}

int PluginServer::close_file()
{
	if(!plugin_open) return 0;
	messages->write_message(CLOSE_FILE);
	return 0;
}

int PluginServer::get_hasaudio()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_HAS_AUDIO);
	return messages->read_message();
}
  
int PluginServer::get_channels()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_CHANNELS);
	return messages->read_message();
}
      
int PluginServer::get_rate()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_RATE);
	return messages->read_message();
}
   	   
int PluginServer::get_bits()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_BITS);
	return messages->read_message();
}
   	   
int PluginServer::get_byteorder()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_BYTE_ORDER);
	return messages->read_message();
}
     
int PluginServer::get_signed()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_SIGNED);
	return messages->read_message();
}
  	   
int PluginServer::get_header()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_HEADER);
	return messages->read_message();
}
  	   
int PluginServer::get_hasvideo()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_HAS_VIDEO);
	return messages->read_message();
}
      
int PluginServer::get_layers()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_LAYERS);
	return messages->read_message();
}
  	   
float PluginServer::get_file_framerate()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_FRAME_RATE);
	return messages->read_message_f();
}

int PluginServer::get_width()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_WIDTH);
	return messages->read_message();
}

int PluginServer::get_height()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_HEIGHT);
	return messages->read_message();
}

int PluginServer::get_quality()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_QUALITY);
	return messages->read_message();
}

int PluginServer::get_compression(char *compression)
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_COMPRESSION);
	return messages->read_message(compression);
}

long PluginServer::get_audio_length()
{
	if(!plugin_open) return 0;
	messages->write_message(FILE_GET_ALENGTH);
	return messages->read_message();
}

long PluginServer::get_video_length()
{
	if(!plugin_open) return -1;
	messages->write_message(FILE_GET_VLENGTH);
	return messages->read_message();
}

int PluginServer::seek_end()
{
	if(!plugin_open) return -1;
	messages->write_message(FILE_SEEK_END);
	return messages->read_message();
}

int PluginServer::seek_start()
{
	if(!plugin_open) return -1;
	messages->write_message(FILE_SEEK_START);
	return messages->read_message();
}

long PluginServer::get_video_position()
{
	if(!plugin_open) return -1;
	messages->write_message(FILE_GET_VIDEO_POSITION);
	return messages->read_message();
}

long PluginServer::get_audio_position()
{
	if(!plugin_open) return -1;
	messages->write_message(FILE_GET_AUDIO_POSITION);
	return messages->read_message();
}

int PluginServer::set_video_position(long frame)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_SET_VIDEO_POSITION);
	messages->write_message(frame);
	return messages->read_message();
}

int PluginServer::set_audio_position(long sample)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_SET_AUDIO_POSITION);
	messages->write_message(sample);
	return messages->read_message();
}

int PluginServer::set_channel(int channel)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_SET_CHANNEL);
	messages->write_message(channel);
	return messages->read_message();
}

int PluginServer::set_layer(int layer)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_SET_LAYER);
	messages->write_message(layer);
	return messages->read_message();
}

int PluginServer::read_samples(PluginBuffer *buffer, long offset, long len, int feather, 
	long lfeather_len, float lfeather_gain, float lfeather_slope, 
	long rfeather_len, float rfeather_gain, float rfeather_slope)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_READ_SAMPLES);
	messages->write_message(buffer->get_id(), buffer->get_size(), offset, len);
	messages->write_message(feather, lfeather_len, rfeather_len);
	messages->write_message_f(lfeather_gain, lfeather_slope);
	messages->write_message_f(rfeather_gain, rfeather_slope);
	return messages->read_message();
}


int PluginServer::read_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset, 
	float in_x1, float in_y1, float in_x2, float in_y2,
	float out_x1, float out_y1, float out_x2, float out_y2, 
	long alpha, long use_alpha, long use_float, long interpolate)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_READ_FRAME);
	messages->write_message(buffer->get_id(), buffer->get_size(), byte_offset);
	messages->write_message_f(in_x1, in_y1, in_x2, in_y2);
	messages->write_message_f(out_x1, out_y1, out_x2, out_y2);
	messages->write_message(alpha, use_alpha, use_float, interpolate);
	messages->write_message(frame->get_w(), frame->get_h());
	return messages->read_message();
}

VFrame* PluginServer::read_frame(int use_alpha, int use_float)
{
	long shared_id, shared_size;
	long width, height;
	int result;

	if(!plugin_open) return 0;
	messages->write_message(FILE_READ_FRAME_PTR);
	messages->write_message(use_alpha, use_float);
	messages->read_message(&width, &height, &shared_id, &shared_size);
	
	if(!temp_frame)
	{
		temp_frame_buffer = new PluginBuffer(shared_id, shared_size, sizeof(VPixel));
		temp_frame = new VFrame((VPixel*)temp_frame_buffer->get_data(), width, height, 1);
	}
	
	result = messages->read_message();
	return temp_frame;
}

int PluginServer::read_raw_frame_possible()
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_RAW_FRAME_POSSIBLE);
	return messages->read_message();
}

int PluginServer::read_raw_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset)
{
	if(!plugin_open) return 1;
	messages->write_message(FILE_READ_RAW_FRAME);
	messages->write_message(buffer->get_id(), buffer->get_size(), byte_offset);
	return messages->read_message();
}

