#ifndef PREFERENCES_H
#define PREFERENCES_H


#define PREFSUBWINDOW_W 440
#define PREFSUBWINDOW_H 350

class CategoryWindow;
class CommandWindow;

#include "bcbase.h"
#include "indexprefs.inc"
#include "mainwindow.inc"
#include "playbackprefs.inc"
#include "pluginprefs.inc"
#include "recordprefs.inc"
#include "thread.h"
#include "viewprefs.inc"
#include "videoprefs.inc"

class Preferences : public BC_MenuItem, public Thread
{
public:
	Preferences(MainWindow *mwindow);
	int handle_event();
	void run();
	
	int default_dialog;
	MainWindow *mwindow;
};

class PreferencesWindow : public BC_Window
{
public:
	PreferencesWindow(char *display = "");
	~PreferencesWindow();
	
	int create_objects(MainWindow *mwindow, Preferences *thread);
	int save_values();
	int delete_current_dialog();
	int set_current_dialog(int number, int first_time = 0);
	
	MainWindow *mwindow;

// ================================ copied values
// ======================================== performance

	char index_directory[1024];   // directory to look in for indexes
	long index_size;                  // size in bytes
	int index_count;
	long cache_size;                  // size in bytes
	int smp;                          // Optimize for smp

// ====================================== recording

	char record_device[1024];
	char video_record_device[1024];
	int record_bitrate, record_channels;
	char duplex_device[1024];
	int duplex_bitrate, duplex_channels;
	int enable_duplex;
	float min_record_db;
	long record_write_length;
	int real_time_record;

// ====================================== playback

	char playback_device[1024];
	char video_playback_device[1024];
	int playback_bitrate, playback_channels;
	int view_follows_playback;
	int real_time_playback;
	long playback_preload;
	long playback_buffer;
	int playback_software_timer;
	long playback_read_length;
	long playback_module_fragment;
	int test_playback_edits;

// ======================================= video

	int video_every_frame;  // play every frame
	int video_output_length;  // number of frames to send through console and output at a time
	int video_read_length;    // number of frames to read from disk during playback.
	int video_write_length;   // number of frames to write to disk at a time during video recording.
	int video_capture_length;   // number of frames to read from device during video recording.
	float actual_frame_rate;
	int video_floatingpoint;  // use floating point math for video
	int video_use_alpha;   // process alpha channels
	int video_interpolate;   // Interpolate video pixels when scaling
	int video_driver;

// ====================================== interface

	int time_format;     // 0 HMS   1 HMSF
	int vu_format;
	int edit_handle_mode[3];           // Edit mode to use for each mouse button
	float frames_per_foot;

// ====================================== plugins

	char global_plugin_dir[1024];
	char local_plugin_dir[1024];

	CategoryWindow *categorywindow;
	CommandWindow *commandwindow;
	
	int need_new_indexes;
	PlaybackPrefs *playbackprefs;
	RecordPrefs *recordprefs;
	IndexPrefs *indexprefs;
	ViewPrefs *viewprefs;
	PluginPrefs *pluginprefs;
	VideoPrefs *videoprefs;
	
	Preferences *thread;
};

class PlaybackButton;
class RecordButton;
class IndexButton;
class ViewButton;
class PluginPrefsButton;
class VideoPrefsButton;

class CategoryWindow : public BC_SubWindow
{
public:
	CategoryWindow(BC_Window *window, PreferencesWindow *pwindow);
	~CategoryWindow();
	
	int create_objects();
	PreferencesWindow *pwindow;
	
	PlaybackButton *playbackbutton;
	RecordButton *recordbutton;
	IndexButton *indexbutton;
	ViewButton *viewbutton;
	PluginPrefsButton *pluginbutton;
};

class OkButton;
class PrefCancelButton;

class CommandWindow : public BC_SubWindow
{
public:
	CommandWindow(BC_Window *window, PreferencesWindow *pwindow);
	~CommandWindow();
	
	int create_objects();
	PreferencesWindow *pwindow;
	OkButton *ok;
	PrefCancelButton *cancel;
};

class OkButton : public BC_BigButton
{
public:
	OkButton(PreferencesWindow *pwindow);
	~OkButton();
	
	int handle_event();
	int keypress_event();
	
	PreferencesWindow *pwindow;
};

class PrefCancelButton : public BC_BigButton
{
public:
	PrefCancelButton(PreferencesWindow *pwindow);
	~PrefCancelButton();
	
	int handle_event();
	int keypress_event();
	
	PreferencesWindow *pwindow;
};

class PlaybackButton : public BC_BigButton
{
public:
	PlaybackButton(PreferencesWindow *pwindow);
	~PlaybackButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

class RecordButton : public BC_BigButton
{
public:
	RecordButton(PreferencesWindow *pwindow);
	~RecordButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

class IndexButton : public BC_BigButton
{
public:
	IndexButton(PreferencesWindow *pwindow);
	~IndexButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

class ViewButton : public BC_BigButton
{
public:
	ViewButton(PreferencesWindow *pwindow);
	~ViewButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

class PluginPrefsButton : public BC_BigButton
{
public:
	PluginPrefsButton(PreferencesWindow *pwindow);
	~PluginPrefsButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

class VideoPrefsButton : public BC_BigButton
{
public:
	VideoPrefsButton(PreferencesWindow *pwindow);
	~VideoPrefsButton();
	int handle_event();
	PreferencesWindow *pwindow;
};

#endif
