#include "channelpicker.h"
#include "mainwindow.h"
#include "record.h"
#include "recordgui.h"
#include "recordtransport.h"
#include "recvideowindow.h"
#include "vframe.h"

RecordVideoWindow::RecordVideoWindow(RecordEngine *engine,Record *record, RecordGUI *gui)
 : Thread(1)
{
	this->record = record;
	this->gui = gui;
	this->engine = engine;
}

RecordVideoWindow::~RecordVideoWindow()
{
	window->set_done(0);
	Thread::join();
	delete window;
}

int RecordVideoWindow::create_objects()
{
	int w, h;
	w = record->frame_w;
	h = record->frame_h;
	fix_size(w, h, record->video_window_w, record->get_aspect_ratio());
	window = new RecordVideoWindowGUI(record, this, w, h + get_buttonbar_height());
	window->create_objects();
	start();
}

void RecordVideoWindow::run()
{
	window->canvas->start_video();
	window->run_window();
	window->canvas->stop_video();
	close_threads();
}

int RecordVideoWindow::close_threads()
{
	window->channel_picker->close_threads();
}

int RecordVideoWindow::update(VFrame *vframe)
{
	window->lock_window();
	window->create_bitmap();

	if(vframe->is_alpha())
		window->bitmap->read_frame(vframe, 0, 0, record->frame_w, record->frame_h, 0);
	else
		window->bitmap->read_frame(vframe->get_rows_noalpha(), 
			record->frame_w, 
			record->frame_h);

	window->canvas->draw_bitmap(window->bitmap, 1);
	window->unlock_window();
}

int RecordVideoWindow::get_buttonbar_height()
{
	return RECBUTTON_HEIGHT;
}

int RecordVideoWindow::fix_size(int &w, int &h, int width_given, float aspect_ratio)
{
	w = width_given;
	h = (int)((float)width_given / aspect_ratio);
}

float RecordVideoWindow::get_scale(int w)
{
	if(record->get_aspect_ratio() > (float)record->frame_w / record->frame_h)
	{
		return (float)w / ((float)record->frame_h * record->get_aspect_ratio());
	}
	else
	{
		return (float)w / record->frame_w;
	}
}

int RecordVideoWindow::get_canvas_height()
{
	return window->get_h() - get_buttonbar_height();
}

int RecordVideoWindow::get_channel_x()
{
//	return 240;
	return 5;
}

int RecordVideoWindow::get_channel_y()
{
	return 2;
}

RecordVideoWindowGUI::RecordVideoWindowGUI(Record *record, 
	RecordVideoWindow *thread, 
	int w, 
	int h)
 : BC_Window(ICONNAME ": Video in", 
 			w, 
 			h, 
			10, 10, 1, 
			!thread->record->video_window_open)
{
	this->thread = thread;
	this->record = record;
	bitmap = 0;
	translating = 0;
	translate_zoom = 0;
}

RecordVideoWindowGUI::~RecordVideoWindowGUI()
{
	if(bitmap) delete bitmap;
}

int RecordVideoWindowGUI::create_objects()
{
	add_tool(canvas = new BC_Canvas(0, thread->get_buttonbar_height(), get_w(), thread->get_canvas_height(), BLACK));
	add_subwindow(buttonbar = new BC_SubWindow(0, 0, get_w(), thread->get_buttonbar_height()));
	buttonbar->add_border();
	record_transport = new RecordGUITransport(thread->record, thread->engine, buttonbar);
	record_transport->create_objects(135, 2, thread->get_buttonbar_height() - 4);
	channel_picker = new ChannelPicker(thread->engine, 
		&(thread->record->mwindow->channeldb), 
		buttonbar);
	channel_picker->create_objects(thread->get_channel_x(), 
		thread->get_channel_y(), 
		thread->get_buttonbar_height());
	set_title();
}

int RecordVideoWindowGUI::button_press()
{
	if(get_buttonpress() == 2)
	{
// Middle button pressed
		if(shift_down())
		{
			translate_zoom = 1;
			get_virtual_center();
		}
		else
		{
			translating = 1;
			get_virtual_center();
		}
		return 1;
	}
	return 0;
}

int RecordVideoWindowGUI::button_release()
{
	if(translating || translate_zoom)
	{
		translating = 0;
		translate_zoom = 0;
		return 1;
	}
	return 0;
}

int RecordVideoWindowGUI::get_virtual_center()
{
	if(translate_zoom)
	{
		virtual_center_y = (int)(get_cursor_y() - record->video_zoom * record->frame_h);
	}
	else
	{
		virtual_center_x = get_cursor_x() - record->video_x;
		virtual_center_y = get_cursor_y() - record->video_y;
	}
}

int RecordVideoWindowGUI::cursor_motion()
{
	if(translating || translate_zoom)
	{
		if(shift_down())
		{
			if(!translate_zoom)
			{
				translate_zoom = 1;
				translating = 0;
				get_virtual_center();
			}
		}
		else
		if(translate_zoom)
		{
			translate_zoom = 0;
			translating = 1;
			get_virtual_center();
		}

		if(translating)
		{
			record->video_x = get_cursor_x() - virtual_center_x;
			record->video_y = get_cursor_y() - virtual_center_y;
			thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
		}

		if(translate_zoom)
		{
			record->video_zoom = (float)(get_cursor_y() - virtual_center_y) / record->frame_h;
			thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
		}
		return 1;
	}
	return 0;
}

int RecordVideoWindowGUI::keypress_event()
{
	int result = 0;
	switch(get_keypress())
	{
		case LEFT:
			if(!ctrl_down()) 
			{ 
				thread->gui->set_translation(--(record->video_x), record->video_y, record->video_zoom);
			}
			else
			{
				record->video_zoom -= 0.1;
				thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
			}
			result = 1;
			break;
		case RIGHT:
			if(!ctrl_down()) 
			{ 
				thread->gui->set_translation(++(record->video_x), record->video_y, record->video_zoom);
			}
			else
			{
				record->video_zoom += 0.1;
				thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
			}
			result = 1;
			break;
		case UP:
			if(!ctrl_down()) 
			{ 
				thread->gui->set_translation(record->video_x, --(record->video_y), record->video_zoom);
			}
			else
			{
				record->video_zoom -= 0.1;
				thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
			}
			result = 1;
			break;
		case DOWN:
			if(!ctrl_down()) 
			{ 
				thread->gui->set_translation(record->video_x, ++(record->video_y), record->video_zoom);
			}
			else
			{
				record->video_zoom += 0.1;
				thread->gui->set_translation(record->video_x, record->video_y, record->video_zoom);
			}
			result = 1;
			break;
		case 'w':
			close_event();
			break;
		default:
			result = record_transport->keypress_event();
			break;
	}
	return result;
}

int RecordVideoWindowGUI::resize_event(int w, int h)
{
	float scale;
	int full_w, full_h;
	int new_w, new_h;

	new_w = w;
	new_h = h;
	if(record->get_aspect_ratio() > (float)record->frame_w / record->frame_h)
	{
		full_w = (int)((float)record->frame_h * record->get_aspect_ratio() + 0.5);
		full_h = record->frame_h;
	}
	else
	{
		full_w = record->frame_w;
		full_h = (int)((float)record->frame_w / record->get_aspect_ratio() + 0.5);
	}

	if(labs(full_w - new_w) < 50)
	{
		new_w = full_w;
		new_h = full_h;
	}
	else
		thread->fix_size(new_w, new_h, w, record->get_aspect_ratio());

	if(new_w < 10) new_w = 10;
	if(new_h < 10) new_h = 10;
	w = record->video_window_w = new_w;
	h = new_h;

	resize_window(new_w, new_h + thread->get_buttonbar_height());
	buttonbar->resize_window(0, 0, new_w, thread->get_buttonbar_height());
	canvas->set_size(0, thread->get_buttonbar_height(), new_w, new_h);
	set_title();
}

int RecordVideoWindowGUI::set_title()
{
	char string[1024];
	int scale;

	scale = (int)(thread->get_scale(thread->record->video_window_w) * 100 + 0.5);

	sprintf(string, ICONNAME ": Video in %d%%", scale);
	BC_Window::set_title(string);
}

int RecordVideoWindowGUI::close_event()
{
	thread->record->monitor_video = 0;
	thread->record->video_window_open = 0;
	thread->gui->monitor_video_toggle->update(0);
	hide_window();
}

int RecordVideoWindowGUI::create_bitmap()
{
	if(bitmap && (bitmap->get_w() != get_w() || bitmap->get_h() != thread->get_canvas_height()))
	{
		delete bitmap;
		bitmap = 0;
	}

	if(!bitmap)
	{
		bitmap = canvas->new_bitmap(get_w(), thread->get_canvas_height());
	}
}
