#include "mainwindow.h"
#include "preferences.h"
#include "videodevice.inc"
#include "videoprefs.h"

VideoPrefs::VideoPrefs(PreferencesWindow *pwindow)
 : BC_SubWindow(150, 10, PREFSUBWINDOW_W, PREFSUBWINDOW_H, MEGREY)
{ this->pwindow = pwindow; }

VideoPrefs::~VideoPrefs()
{
}

int VideoPrefs::create_objects()
{
	char string[1024];

	add_border(get_resources()->get_bg_shadow1(),
		get_resources()->get_bg_shadow2(),
		get_resources()->get_bg_color(),
		get_resources()->get_bg_light2(),
		get_resources()->get_bg_light1());
	int y = 5;
	add_tool(new BC_Title(10, y, "Video Preferences", LARGEFONT, RED));
	y += 30;
	add_tool(new BC_Title(10, y, "Actual framerate being achieved:"));
	add_tool(frame_rate = new BC_Title(250, y, "-", MEDIUMFONT, YELLOW));
	draw_framerate();
	y += 25;
	add_tool(new VideoEveryFrame(pwindow, y));
//	y += 35;
//	add_tool(new BC_Title(5, y, "Frames to read from disk at a time:"));
//	y += 25;
//	add_tool(new VideoReadLength(pwindow, y));
//	y += 30;
//	add_tool(new BC_Title(5, y, "Frames to send through console at a time:"));
//	y += 25;
//	add_tool(new VideoOutputLength(pwindow, y));
	y += 20;
	add_tool(fastest = new VideoFastest(pwindow, y, this));
	y += 20;
	add_tool(alphachannels = new VideoAlphaChannels(pwindow, y, this));
	y += 20;
	add_tool(floatingpoint = new VideoFloatingPoint(pwindow, y, this));
	y += 20;
	add_tool(interpolate = new VideoInterpolate(pwindow, y, this));
	y += 25;
	add_tool(new BC_Title(10, y, "Record Device:"));
	add_tool(new VideoRecordDevice(pwindow, y + 20));
	//y += 30;
	//add_tool(new BC_Title(10, y, "Playback Device:"));
	//y += 25;
	//add_tool(new VideoPlaybackDevice(pwindow, y));
	add_tool(new BC_Title(200, y, "Record driver:"));
	switch(pwindow->video_driver)
	{
		case VIDEO4LINUX:
			sprintf(string, "Video4linux");
			break;
		case VIDEO4LINUX2:
			sprintf(string, "Video4linux2");
			break;
		case SCREENCAPTURE:
			sprintf(string, "Screen Capture");
			break;
	}
	add_tool(new VideoDriverText(200, y + 20, string, pwindow));
	y += 50;
	sprintf(string, "%d", pwindow->video_write_length);
	add_tool(new VideoWriteLength(pwindow, string, y));
	add_tool(new BC_Title(120, y, "Frames to record to disk at a time."));
	y += 30;
	sprintf(string, "%d", pwindow->video_capture_length);
	add_tool(new VideoCaptureLength(pwindow, string, y));
	add_tool(new BC_Title(120, y, "Frames to buffer in device."));
}

int VideoPrefs::set_optimizations(int fastest, int alphachannels, int floatingpoint, int interpolate)
{
	pwindow->video_interpolate = interpolate;
	pwindow->video_floatingpoint = floatingpoint | interpolate;
	pwindow->video_use_alpha = alphachannels | floatingpoint | interpolate;
	this->fastest->update(fastest);
	this->alphachannels->update(alphachannels);
	this->floatingpoint->update(floatingpoint);
	this->interpolate->update(interpolate);
}

int VideoPrefs::draw_framerate()
{
	char string[1024];
	if(pwindow->actual_frame_rate <= 0)
		sprintf(string, "%.2f fps", pwindow->mwindow->frame_rate);
	else
		sprintf(string, "%.2f fps", pwindow->actual_frame_rate);
	frame_rate->update(string);
}


VideoEveryFrame::VideoEveryFrame(PreferencesWindow *pwindow, int y)
 : BC_CheckBox(10, y, 16, 16, pwindow->video_every_frame, "Play every frame")
{ this->pwindow = pwindow; }

VideoEveryFrame::~VideoEveryFrame()
{
}

int VideoEveryFrame::handle_event()
{
	pwindow->video_every_frame = get_value();
}




VideoReadLength::VideoReadLength(PreferencesWindow *pwindow, int y)
 : BC_TextBox(5, y, 100, pwindow->video_read_length)
{ this->pwindow = pwindow; }

VideoReadLength::~VideoReadLength()
{
}

int VideoReadLength::handle_event()
{
	pwindow->video_read_length = atol(get_text());
}




VideoOutputLength::VideoOutputLength(PreferencesWindow *pwindow, int y)
 : BC_TextBox(5, y, 100, pwindow->video_output_length)
{ this->pwindow = pwindow; }

VideoOutputLength::~VideoOutputLength()
{
}

int VideoOutputLength::handle_event()
{
	pwindow->video_output_length = atol(get_text());
}


VideoRateReset::VideoRateReset(PreferencesWindow *pwindow, int y, VideoPrefs *prefs)
 : BC_BigButton(100, y, "Reset")
{
	this->pwindow = pwindow; this->prefs = prefs;
}

VideoRateReset::~VideoRateReset()
{
}

int VideoRateReset::handle_event()
{
	pwindow->actual_frame_rate = -1;
	prefs->draw_framerate();
}

VideoInterpolate::VideoInterpolate(PreferencesWindow *pwindow, int y, VideoPrefs *prefs)
 : BC_Radial(10, y, 16, 16, pwindow->video_interpolate, "Interpolated, floating point alpha channels")
{ this->pwindow = pwindow; this->prefs = prefs; }

VideoInterpolate::~VideoInterpolate()
{
}

int VideoInterpolate::handle_event()
{	
	update(1);
	prefs->set_optimizations(0, 0, 0, 1);
}


VideoFloatingPoint::VideoFloatingPoint(PreferencesWindow *pwindow, int y, VideoPrefs *prefs)
 : BC_Radial(10, y, 16, 16, 
 	pwindow->video_floatingpoint && !(pwindow->video_interpolate), "Floating point alpha channels")
{ this->pwindow = pwindow; this->prefs = prefs; }

VideoFloatingPoint::~VideoFloatingPoint()
{
}

int VideoFloatingPoint::handle_event()
{
	update(1);
	prefs->set_optimizations(0, 0, 1, 0);
}


VideoAlphaChannels::VideoAlphaChannels(PreferencesWindow *pwindow, int y, VideoPrefs *prefs)
 : BC_Radial(10, y, 16, 16, 
 	pwindow->video_use_alpha && !(pwindow->video_floatingpoint | pwindow->video_interpolate), 
 	"Alpha channels")
{ this->pwindow = pwindow; this->prefs = prefs; }

VideoAlphaChannels::~VideoAlphaChannels()
{
}

int VideoAlphaChannels::handle_event()
{
	update(1);
	prefs->set_optimizations(0, 1, 0, 0);
}

VideoFastest::VideoFastest(PreferencesWindow *pwindow, int y, VideoPrefs *prefs)
 : BC_Radial(10, y, 16, 16, 
 	!(pwindow->video_use_alpha | pwindow->video_floatingpoint | pwindow->video_interpolate), 
 	"Fastest rendering")
{ this->pwindow = pwindow; this->prefs = prefs; }

VideoFastest::~VideoFastest()
{
}

int VideoFastest::handle_event()
{
	update(1);
	prefs->set_optimizations(1, 0, 0, 0);
}


VideoPlaybackDevice::VideoPlaybackDevice(PreferencesWindow *pwindow, int y)
 : BC_TextBox(10, y, 150, pwindow->video_playback_device)
{
	this->pwindow = pwindow;
}

VideoPlaybackDevice::~VideoPlaybackDevice()
{
}

int VideoPlaybackDevice::handle_event()
{
	strcpy(pwindow->video_playback_device, get_text());
}

VideoRecordDevice::VideoRecordDevice(PreferencesWindow *pwindow, int y)
 : BC_TextBox(10, y, 150, pwindow->video_record_device)
{
	this->pwindow = pwindow;
}

VideoRecordDevice::~VideoRecordDevice()
{
}

int VideoRecordDevice::handle_event()
{
	strcpy(pwindow->video_record_device, get_text());
}


VideoDriverText::VideoDriverText(int x, int y, char *text, PreferencesWindow *pwindow)
 : BC_PopupMenu(x, y, 125, text)
{
	this->pwindow = pwindow;
}

VideoDriverText::~VideoDriverText()
{
}

int VideoDriverText::handle_event()
{
}

int VideoDriverText::add_items()
{
// Video4linux versions are automatically detected
	add_item(new VideoDriverItem(this, "Video4Linux", VIDEO4LINUX));
//	add_item(new VideoDriverItem(this, "Video4Linux2", VIDEO4LINUX2));
	add_item(new VideoDriverItem(this, "Screen Capture", SCREENCAPTURE));
}


VideoDriverItem::VideoDriverItem(VideoDriverText *popup, char *text, int driver)
 : BC_PopupItem(text)
{
	this->popup = popup;
	this->driver = driver;
}

VideoDriverItem::~VideoDriverItem()
{
}

int VideoDriverItem::handle_event()
{
	popup->update(get_text());
	popup->pwindow->video_driver = driver;
}



VideoWriteLength::VideoWriteLength(PreferencesWindow *pwindow, char *text, int y)
 : BC_TextBox(10, y, 100, text)
{ this->pwindow = pwindow; }

int VideoWriteLength::handle_event()
{ pwindow->video_write_length = atol(get_text()); }

VideoCaptureLength::VideoCaptureLength(PreferencesWindow *pwindow, char *text, int y)
 : BC_TextBox(10, y, 100, text)
{ this->pwindow = pwindow; }

int VideoCaptureLength::handle_event()
{ pwindow->video_capture_length = atol(get_text()); }
