#include "pluginioclient.h"
#include "vframe.h"


PluginIOClient::PluginIOClient(int argc, char *argv[])
 : PluginClient(argc, argv)
{
	compression[0] = 0;
}

PluginIOClient::~PluginIOClient()
{
}

int PluginIOClient::plugin_is_fileio()
{
	return 1;
}

int PluginIOClient::plugin_command_derived(int plugin_command)
{
	switch(plugin_command)
	{
		case OPEN_FILE:               plugin_open_file();		    	break;
		case CLOSE_FILE:              plugin_close_file();  	    	break;
		case CHECK_HEADER:            plugin_check_header();	    	break;
		case GET_AUDIO_PARAMETERS:    plugin_get_aparameters();     	break;
		case GET_VIDEO_PARAMETERS:    plugin_get_vparameters();     	break;
		case FILE_GET_HAS_AUDIO:      plugin_get_hasaudio();	    	break;
		case FILE_GET_CHANNELS:       plugin_get_channels();	    	break;
		case FILE_GET_RATE:           plugin_get_rate();		    	break;
		case FILE_GET_BITS:           plugin_get_bits();		    	break;
		case FILE_GET_BYTE_ORDER:     plugin_get_byteorder();	    	break;
		case FILE_GET_SIGNED:         plugin_get_signed();  	    	break;
		case FILE_GET_HEADER:         plugin_get_header();  	    	break;
		case FILE_GET_HAS_VIDEO:      plugin_get_hasvideo();	    	break;
		case FILE_GET_LAYERS:         plugin_get_layers();  	    	break;
		case FILE_GET_FRAME_RATE:     plugin_get_framerate();	    	break;
		case FILE_GET_WIDTH:          plugin_get_width();		    	break;
		case FILE_GET_HEIGHT:         plugin_get_height();  	    	break;
		case FILE_GET_QUALITY:        plugin_get_quality(); 	    	break;
		case FILE_GET_COMPRESSION:    plugin_get_compression();     	break;
		case FILE_GET_ALENGTH:        plugin_get_alength(); 	    	break;
		case FILE_GET_VLENGTH:        plugin_get_vlength(); 	    	break;
		case FILE_SEEK_END:           plugin_seek_end();                break;
		case FILE_SEEK_START:         plugin_seek_start();              break;
		case FILE_GET_VIDEO_POSITION: plugin_get_vposition();           break;
		case FILE_GET_AUDIO_POSITION: plugin_get_aposition();           break;
		case FILE_SET_VIDEO_POSITION: plugin_set_vposition();           break;
		case FILE_SET_AUDIO_POSITION: plugin_set_aposition();           break;
		case FILE_SET_CHANNEL:        plugin_set_channel();             break;
		case FILE_SET_LAYER:          plugin_set_layer();               break;
		case FILE_READ_SAMPLES:       plugin_read_samples();            break;
		case FILE_READ_FRAME:         plugin_read_frame();              break;
		case FILE_READ_FRAME_PTR:     plugin_read_frame_ptr();          break;
		case FILE_RAW_FRAME_POSSIBLE: plugin_raw_frame_possible();      break;
		case FILE_READ_RAW_FRAME:     plugin_read_raw_frame();          break;
	}
}

int PluginIOClient::plugin_check_header()
{
	char path[1024];
	messages->read_message(path);
	int result = check_header(path);
	send_completed();
	messages->write_message(result);
}

int PluginIOClient::plugin_open_file()
{
	char path[1024];
	long rd_l, wr_l;
	messages->read_message(path);
	messages->read_message(&rd_l, &wr_l);
	wr = wr_l;
	rd = rd_l;

// Reset parameters
	has_audio = 0;
	sample_rate = 0;
	channels = 0;
	bits = 0;
	byteorder = 0;
	signed_ = 0;
	header = 0;
	
	has_video = 0;
	layers = 0;
	frame_rate = 0;
	width = 0;
	height = 0;
	quality = 0;
	compression[0] = 0;

// Can't get parameters from server because the server doesn't have mainwindow.

// Open it for real
	int result = open_file(path, rd, wr);
	send_completed();
	messages->write_message(result);
}

int PluginIOClient::plugin_close_file()
{
	close_file();
}

int PluginIOClient::plugin_get_aparameters()
{
	int result = get_audio_parameters();
	send_completed();
	messages->write_message(result);
}

int PluginIOClient::plugin_get_vparameters()
{
	int result = get_video_parameters();
	send_completed();
	messages->write_message(result);
}

int PluginIOClient::plugin_get_hasaudio() { messages->write_message(has_audio); }
int PluginIOClient::plugin_get_channels() { messages->write_message(channels); }    
int PluginIOClient::plugin_get_rate() { messages->write_message(sample_rate); }        
int PluginIOClient::plugin_get_bits() { messages->write_message(bits); }        
int PluginIOClient::plugin_get_byteorder() { messages->write_message(byteorder); }   
int PluginIOClient::plugin_get_signed() { messages->write_message(signed_); }      
int PluginIOClient::plugin_get_header() { messages->write_message(header); }      
int PluginIOClient::plugin_get_hasvideo() { messages->write_message(has_video); }    
int PluginIOClient::plugin_get_layers() { messages->write_message(layers); }      
float PluginIOClient::plugin_get_framerate() { messages->write_message_f(frame_rate); }   
int PluginIOClient::plugin_get_width() { messages->write_message(width); }       
int PluginIOClient::plugin_get_height() { messages->write_message(height); }      
int PluginIOClient::plugin_get_quality() { messages->write_message(quality); }     
int PluginIOClient::plugin_get_compression() { messages->write_message(compression); } 
long PluginIOClient::plugin_get_alength() { messages->write_message(get_alength()); }
long PluginIOClient::plugin_get_vlength() { messages->write_message(get_vlength()); }
int PluginIOClient::plugin_seek_end() { messages->write_message(seek_end()); }
int PluginIOClient::plugin_seek_start() { messages->write_message(seek_start()); }
long PluginIOClient::plugin_get_vposition() { messages->write_message(get_vposition()); }
long PluginIOClient::plugin_get_aposition() { messages->write_message(get_aposition()); }
int PluginIOClient::plugin_set_vposition() { messages->write_message(set_vposition(messages->read_message())); }
int PluginIOClient::plugin_set_aposition() { messages->write_message(set_aposition(messages->read_message())); }
int PluginIOClient::plugin_set_channel() { messages->write_message(set_channel(messages->read_message())); }
int PluginIOClient::plugin_set_layer() { messages->write_message(set_layer(messages->read_message())); }
int PluginIOClient::plugin_read_samples()
{
	int result = 0;
	long shared_id;
	long shared_size;
	long shared_offset;
	PluginBuffer *shared_buffer;
	float *buffer;
	long len;
	long feather;
	long lfeather_len;
	float lfeather_gain;
	float lfeather_slope; 
	long rfeather_len;
	float rfeather_gain;
	float rfeather_slope;

	messages->read_message(&shared_id, &shared_size, &shared_offset, &len);
	messages->read_message(&feather, &lfeather_len, &rfeather_len);
	messages->read_message_f(&lfeather_gain, &lfeather_slope);
	messages->read_message_f(&rfeather_gain, &rfeather_slope);

	shared_buffer = new PluginBuffer(shared_id, shared_size, sizeof(float));
	buffer = (float*)shared_buffer->get_data() + shared_offset;
	result = read_samples(buffer, len, 
		feather, 
		lfeather_len, lfeather_gain, lfeather_slope, 
		rfeather_len, rfeather_gain, rfeather_slope);

	delete shared_buffer;
	messages->write_message(result);
	return 0;
}

int PluginIOClient::plugin_read_frame()
{
	long shared_id, shared_size, shared_offset;
	float in_x1, in_y1, in_x2, in_y2;
	float out_x1, out_y1, out_x2, out_y2;
	long alpha, use_alpha, use_float, interpolate;
	long frame_w, frame_h;

	messages->read_message(&shared_id, &shared_size, &shared_offset);
	messages->read_message_f(&in_x1, &in_y1, &in_x2, &in_y2);
	messages->read_message_f(&out_x1, &out_y1, &out_x2, &out_y2);
	messages->read_message(&alpha, &use_alpha, &use_float, &interpolate);
	messages->read_message(&frame_w, &frame_h);

	PluginBuffer buffer(shared_id, shared_size, sizeof(VPixel));
	VFrame frame((VPixel*)((char*)buffer.get_data() + shared_offset), frame_w, frame_h, 1);
	int result = read_frame(&frame, 
		in_x1, in_y1, in_x2, in_y2,
		out_x1, out_y1, out_x2, out_y2, 
		alpha, use_alpha, use_float, interpolate);

	messages->write_message(result);
	return 0;
}

int PluginIOClient::plugin_read_frame_ptr()
{
	long use_alpha, use_float;
	long shared_id, shared_size;
	int result;
	messages->read_message(&use_alpha, &use_float);

	result = read_frame_ptr(use_alpha, use_float, shared_id, shared_size);
	messages->write_message(width, height, shared_id, shared_size);
	messages->write_message(result);
	return 0;
}

int PluginIOClient::plugin_raw_frame_possible()
{
	messages->write_message(read_raw_frame_possible());
	return 0;
}

int PluginIOClient::plugin_read_raw_frame()
{
	long shared_id, shared_size, shared_offset;

	messages->read_message(&shared_id, &shared_size, &shared_offset);

	PluginBuffer buffer(shared_id, shared_size, 3);
	VFrame frame((unsigned char*)buffer.get_data() + shared_offset, width, height);
	int result = read_raw_frame(&frame);
	messages->write_message(result);
	return 0;
}
