#include "filehtal.h"
#include "sharpen.h"
#include "sharpenwindow.h"

main(int argc, char *argv[])
{
	SharpenMain *plugin;

	plugin = new SharpenMain(argc, argv);
	plugin->plugin_run();
	delete plugin;
}

SharpenMain::SharpenMain(int argc, char *argv[])
 : PluginVClient(argc, argv)
{
	sharpness = 0;
}

SharpenMain::~SharpenMain()
{
}

char* SharpenMain::plugin_title() { return "Sharpen"; }
int SharpenMain::plugin_is_realtime() { return 1; }
int SharpenMain::plugin_is_multi_channel() { return 0; }

int SharpenMain::load_defaults()
{
	char directory[1024], string[1024];
// set the default directory
	sprintf(directory, "%ssharpen.rc", BCASTDIR);

// load the defaults
	defaults = new Defaults(directory);
	defaults->load();

	sharpness = defaults->get("SHARPNESS", 50);
}

int SharpenMain::save_defaults()
{
	defaults->update("SHARPNESS", sharpness);
	defaults->save();
}

int SharpenMain::start_realtime()
{
// Initialize
	get_luts(pos_lut, neg_lut);
	for(int i = 0; i < 4; i++)
	{
		neg_rows[i] = new int[project_frame_w * 4];
	}
	last_sharpness = sharpness;
}

int SharpenMain::stop_realtime()
{
	for(int i = 0; i < 4; i++)
	{
		delete neg_rows[i];
	}
}

int SharpenMain::process_realtime(long size, VFrame **input_ptr, VFrame **output_ptr)
{
	register int i, j, k, row, count;
	VPixel **input_rows, **output_rows;
	float old_sharpness = sharpness;

	for(i = 0; i < size; i++)
	{
		input_rows = input_ptr[i]->get_rows();
		output_rows = output_ptr[i]->get_rows();
		sharpness = old_sharpness;

		if(automation_used())
			sharpness += (get_automation_value(i) * MAXSHARPNESS);

		if(sharpness > MAXSHARPNESS) sharpness = MAXSHARPNESS;
		else
			if(sharpness < 0) sharpness = 0;

		if(last_sharpness != sharpness) get_luts(pos_lut, neg_lut);
		if(sharpness != 0)
		{
// Arm first row
			src_rows[0] = input_rows[0];
			for(j = 0; j < project_frame_w; j++)
			{
				neg_rows[0][j * 4] = neg_lut[src_rows[0][j].red];
				neg_rows[0][j * 4 + 1] = neg_lut[src_rows[0][j].green];
				neg_rows[0][j * 4 + 2] = neg_lut[src_rows[0][j].blue];
				neg_rows[0][j * 4 + 3] = neg_lut[src_rows[0][j].alpha];
			}

			row = 1;
			count = 1;

			for(i = 0; i < project_frame_h; i++)
			{
				if((i + 1) < project_frame_h)
				{
					if(count >= 3) count--;
// Arm next row
					src_rows[row] = input_rows[i + 1];
					for(k = 0; k < project_frame_w; k++)
					{
						neg_rows[row][k * 4] = neg_lut[src_rows[row][k].red];
						neg_rows[row][k * 4 + 1] = neg_lut[src_rows[row][k].green];
						neg_rows[row][k * 4 + 2] = neg_lut[src_rows[row][k].blue];
						neg_rows[row][k * 4 + 3] = neg_lut[src_rows[row][k].alpha];
					}
					
					count++;
					row = (row + 1) & 3;
				}
				else
				{
					count--;
				}
				
				dst_row = output_rows[i];
				if(count == 3)
				{
// Do the filter
					filter(project_frame_w, src_rows[(row + 2) & 3], dst_row,
						neg_rows[(row + 1) & 3] + 4,
						neg_rows[(row + 2) & 3] + 4,
						neg_rows[(row + 3) & 3] + 4);
				}
				else if(count == 2)
				{
					if(i == 0)
						for(k = 0; k < project_frame_w; k++) dst_row[k] = src_rows[0][k];
					else
						for(k = 0; k < project_frame_w; k++) dst_row[k] = src_rows[2][k];
				}
			}
		}
		else
		if(!buffers_identical(0))
		{
// Data never processed so copy if necessary
			for(j = 0; j < project_frame_h; j++)
			{
				for(k = 0; k < project_frame_w; k++)
				{
					output_rows[j][k] = input_rows[j][k];
				}
			}
		}
		last_sharpness = sharpness;
	}
	sharpness = old_sharpness;
}

int SharpenMain::get_luts(int *pos_lut, int *neg_lut)
{
	int i, inv_sharpness;
	
	inv_sharpness = (int)(100 - sharpness);
	if(inv_sharpness < 1) inv_sharpness = 1;
	for(i = 0; i < VMAX + 1; i++)
	{
		pos_lut[i] = 800 * i / inv_sharpness;
		neg_lut[i] = (4 + pos_lut[i] - (i << 3)) >> 3;
	}
}

int SharpenMain::filter(register int w, VPixel *src, VPixel *dst,
	int *neg0, int *neg1, int *neg2)
{
	int pixel;
	
	*dst++ = *src++;
	w -= 2;
	
	while(w > 0)
	{
		pixel = pos_lut[src->red] - neg0[-4] - neg0[0] - neg0[4] - 
			neg1[-4] - neg1[4] - neg2[-4] - neg2[0] - neg2[4];
		pixel = (pixel + 4) >> 3;
		if(pixel < 0) dst->red = 0;
		else
		if(pixel > VMAX) dst->red = VMAX;
		else
		dst->red = pixel;

		pixel = pos_lut[src->green] - neg0[-3] - neg0[1] - neg0[5] - 
			neg1[-3] - neg1[5] - neg2[-3] - neg2[1] - neg2[5];
		pixel = (pixel + 4) >> 3;
		if(pixel < 0) dst->green = 0;
		else
		if(pixel > VMAX) dst->green = VMAX;
		else
		dst->green = pixel;

		pixel = pos_lut[src->blue] - neg0[-2] - neg0[2] - neg0[6] - 
			neg1[-2] - neg1[6] - neg2[-2] - neg2[2] - neg2[6];
		pixel = (pixel + 4) >> 3;
		if(pixel < 0) dst->blue = 0;
		else
		if(pixel > VMAX) dst->blue = VMAX;
		else
		dst->blue = pixel;

// Don't touch alpha
		*dst = *src;

		src++;
		dst++;
		
		neg0 += 4;
		neg1 += 4;
		neg2 += 4;
		w--;
	}
	
	*dst++ = *src++;
}


int SharpenMain::start_gui()
{
	load_defaults();
	thread = new SharpenThread(this);
	thread->start();
	thread->gui_started.lock();
}

int SharpenMain::stop_gui()
{
	save_defaults();
	thread->window->set_done(0);
	thread->join();
	delete thread;
	thread = 0;
}

int SharpenMain::show_gui()
{
	thread->window->show_window();
}

int SharpenMain::hide_gui()
{
	thread->window->hide_window();
}

int SharpenMain::set_string()
{
	thread->window->set_title(gui_string);
}

int SharpenMain::save_data(char *text)
{
	FileHTAL output;

// cause data to be stored directly in text
	output.set_shared_string(text, MESSAGESIZE);
	output.tag.set_title("SHARPNESS");
	output.tag.set_property("VALUE", sharpness);
	output.append_tag();
	output.terminate_string();
// data is now in *text
}

int SharpenMain::read_data(char *text)
{
	FileHTAL input;

	input.set_shared_string(text, strlen(text));

	int result = 0;

	while(!result)
	{
		result = input.read_tag();

		if(!result)
		{
			if(input.tag.title_is("SHARPNESS"))
			{
				sharpness = input.tag.get_property("VALUE", sharpness);
// You'll usually set a flag here to inform process_realtime
// that it needs to reconfigure itself or something.
// Here we just change the coefficients from under it.
				get_luts(pos_lut, neg_lut);
				last_sharpness = sharpness;
			}
		}
	}
	if(thread) 
	{
		thread->window->sharpen_slider->update((int)sharpness);
	}
}
