/* 
 *
 *	uncouple.c Copyright (C) Aaron Holtzman - May 1999
 *
 *  This file is part of libmpeg2
 *	
 *  libmpeg2 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  libmpeg2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include "../bitstream.h"
#include "mpeg2audio.h"

static unsigned char mpeg2_first_bit_lut[256] = 
{
	0, 8, 7, 7, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
};

/* Converts an unsigned exponent in the range of 0-24 and a 16 bit mantissa
 * to an IEEE single precision floating point value */
void mpeg2audio_ac3_convert_to_float(unsigned short exp, 
	unsigned short mantissa, 
	unsigned MPEG2_INT32 *dest)
{
	int num;
	short exponent;
	char i;

/* If the mantissa is zero we can simply return zero */
	if(mantissa == 0)
	{
		*dest = 0;
		return;
	}

/* Exponent is offset by 127 in IEEE format minus the shift to
 * align the mantissa to 1.f (subtracted in the final result) */
	exponent = 127 - exp;

/* Take care of the one asymmetric negative number */
	if(mantissa == 0x8000)
		mantissa++;

	/* Extract the sign bit, invert the mantissa if it's negative, and 
	   shift out the sign bit */
	if( mantissa & 0x8000 )
	{
		mantissa *= -2;
		num = 0x80000000 + (exponent << 23);
	}
	else
	{
		mantissa *= 2;
		num = exponent << 23;
	}

/* Find the index of the most significant one bit */
	i = mpeg2_first_bit_lut[mantissa >> 8];

	if(i == 0)
		i = mpeg2_first_bit_lut[mantissa & 0xff] + 8;
	
	*dest = num - (i << 23) + (mantissa << (7 + i));
	return;
}


void mpeg2audio_ac3_uncouple_channel(mpeg2audio_t *audio,
		mpeg2_stream_coeffs_t *coeffs,
		mpeg2_ac3audblk_t *audblk, 
		unsigned int ch)
{
	unsigned int bnd = 0;
	unsigned int i, j;
	MPEG2_FLOAT32 cpl_coord;
	MPEG2_FLOAT32 coeff;
	unsigned int cpl_exp_tmp;
	unsigned int cpl_mant_tmp;

	for(i = audblk->cplstrtmant;i<audblk->cplendmant;)
	{
		if(!audblk->cplbndstrc[bnd])
		{
			cpl_exp_tmp = audblk->cplcoexp[ch][bnd] + 3 * audblk->mstrcplco[ch];
			if(audblk->cplcoexp[ch][bnd] == 15)
				cpl_mant_tmp = (audblk->cplcomant[ch][bnd]) << 12;
			else
				cpl_mant_tmp = ((0x10) | audblk->cplcomant[ch][bnd]) << 11;
			
			mpeg2audio_ac3_convert_to_float(cpl_exp_tmp, cpl_mant_tmp, (unsigned MPEG2_INT32*)&cpl_coord);
		}
		bnd++;

		for(j = 0; j < 12; j++)
		{
//Get new dither values for each channel if necessary, so
//the channels are uncorrelated
			if(audblk->dithflag[ch] && audblk->cpl_bap[i] == 0)
				mpeg2audio_ac3_convert_to_float(audblk->cpl_exp[i], mpeg2audio_ac3_dither_gen(audio), (unsigned MPEG2_INT32*)&coeff);
			else
				mpeg2audio_ac3_convert_to_float(audblk->cpl_exp[i], audblk->cplmant[i],(unsigned MPEG2_INT32*)&coeff);

			coeffs->fbw[ch][i]  = cpl_coord * coeff;

			i++;
		}
	}
	return;
}


int mpeg2audio_ac3_uncouple(mpeg2audio_t *audio, 
		mpeg2_ac3bsi_t *bsi,
		mpeg2_ac3audblk_t *audblk,
		mpeg2_stream_coeffs_t *coeffs)
{
	int i, j;

	for(i = 0; i < bsi->nfchans; i++)
	{
		for(j = 0; j < audblk->endmant[i]; j++)
			 mpeg2audio_ac3_convert_to_float(audblk->fbw_exp[i][j],audblk->chmant[i][j],
					 (unsigned MPEG2_INT32*)&coeffs->fbw[i][j]);
	}

	if(audblk->cplinu)
	{
		for(i = 0; i < bsi->nfchans; i++)
		{
			if(audblk->chincpl[i])
			{
				mpeg2audio_ac3_uncouple_channel(audio, 
					coeffs,
					audblk,
					i);
			}
		}

	}

	if(bsi->lfeon)
	{
/* There are always 7 mantissas for lfe */
		for(j = 0; j < 7 ; j++)
			 mpeg2audio_ac3_convert_to_float(audblk->lfe_exp[j], 
			 		audblk->lfemant[j],
					 (unsigned MPEG2_INT32*)&coeffs->lfe[j]);

	}
	return 0;
}
