#include "../libmpeg2.h"
#include "mpeg2video.h"


static long long mpeg2_MMX_0 = 0L;
static unsigned long  mpeg2_MMX_10w[]         = {0x00100010, 0x00100010};                     //dd    00010 0010h, 000100010h
static unsigned long  mpeg2_MMX_80w[]         = {0x00800080, 0x00800080};                     //dd    00080 0080h, 000800080h

static unsigned long  mpeg2_MMX_00FFw[]       = {0x00ff00ff, 0x00ff00ff};                     //dd    000FF 00FFh, 000FF00FFh

static unsigned short mpeg2_MMX_Ublucoeff[]   = {0x81, 0x81, 0x81, 0x81};                     //dd    00081 0081h, 000810081h
static unsigned short mpeg2_MMX_Vredcoeff[]   = {0x66, 0x66, 0x66, 0x66};                     //dd    00066 0066h, 000660066h

static unsigned short mpeg2_MMX_Ugrncoeff[]   = {0xffe8, 0xffe8, 0xffe8, 0xffe8};             //dd    0FFE7 FFE7h, 0FFE7FFE7h
static unsigned short mpeg2_MMX_Vgrncoeff[]   = {0xffcd, 0xffcd, 0xffcd, 0xffcd};             //dd    0FFCC FFCCh, 0FFCCFFCCh

static unsigned short mpeg2_MMX_Ycoeff[]      = {0x4a, 0x4a, 0x4a, 0x4a};                     //dd    0004A 004Ah, 0004A004Ah
static unsigned short mpeg2_MMX_redmask[]     = {0xf800, 0xf800, 0xf800, 0xf800};             //dd    07c00 7c00h, 07c007c00h

static unsigned short mpeg2_MMX_grnmask[]     = {0x7e0, 0x7e0, 0x7e0, 0x7e0};                 //dd    003e0 03e0h, 003e003e0h


// Algorithm
// 			r = (int)(*y + 1.371 * (*cr - 128));
// 			g = (int)(*y - 0.698 * (*cr - 128) - 0.336 * (*cb - 128));
// 			b = (int)(*y + 1.732 * (*cb - 128));

#ifdef HAVE_MMX
inline void mpeg2video_rgb16_mmx(unsigned char *lum, 
			unsigned char *cr, 
			unsigned char *cb,
            unsigned char *out, 
			int rows, 
			int cols, 
			int mod)
{
	unsigned short *row1;
	int x;
    unsigned char *y;
	int col1;

	row1 = (unsigned short *)out;
    col1 = cols + mod;
    mod += cols + mod;
    mod *= 2;
	y = lum + cols * rows;
    x = 0;

    __asm__ __volatile__(
        ".align 8\n"
        "1:\n"
            "movd           (%1),                   %%mm0\n"  // 4 Cb	  0  0  0  0 u3 u2 u1 u0
            "pxor           %%mm7,                  %%mm7\n"
            "movd           (%0),                   %%mm1\n"  // 4 Cr	  0  0  0  0 v3 v2 v1 v0
            "punpcklbw      %%mm7,                  %%mm0\n"  // 4 W cb   0 u3  0 u2  0 u1  0 u0
            "punpcklbw      %%mm7,                  %%mm1\n"  // 4 W cr   0 v3  0 v2  0 v1  0 v0

            "psubw          mpeg2_MMX_80w,          %%mm0\n"
            "psubw          mpeg2_MMX_80w,          %%mm1\n"
            "movq           %%mm0,                  %%mm2\n"  // Cb 	  0 u3  0 u2  0 u1  0 u0
            "movq           %%mm1,                  %%mm3\n"  // Cr
            "pmullw         mpeg2_MMX_Ugrncoeff,    %%mm2\n"  // Cb2green 0 R3  0 R2  0 R1  0 R0
            "movq           (%2),                   %%mm6\n"  // L1 	 l7 L6 L5 L4 L3 L2 L1 L0
            "pmullw         mpeg2_MMX_Ublucoeff,    %%mm0\n"  // Cb2blue
            "pand           mpeg2_MMX_00FFw,        %%mm6\n"  // L1 	 00 L6 00 L4 00 L2 00 L0
            "pmullw         mpeg2_MMX_Vgrncoeff,    %%mm3\n"  // Cr2green
            "movq           (%2),                   %%mm7\n"  // L2
            "pmullw         mpeg2_MMX_Vredcoeff,    %%mm1\n"  // Cr2red
            "psrlw          $8,                     %%mm7\n"  // L2 	 00 L7 00 L5 00 L3 00 L1
            "pmullw         mpeg2_MMX_Ycoeff,       %%mm6\n"  // lum1
            "paddw          %%mm3,                  %%mm2\n"  // Cb2green + Cr2green == green
            "pmullw         mpeg2_MMX_Ycoeff,       %%mm7\n"  // lum2

            "movq           %%mm6,                  %%mm4\n"  // lum1
            "paddw          %%mm0,                  %%mm6\n"  // lum1 +blue  00 B6 00 B4 00 B2 00 B0
            "movq           %%mm4,                  %%mm5\n"  // lum1
            "paddw          %%mm1,                  %%mm4\n"  // lum1 +red   00 R6 00 R4 00 R2 00 R0
            "paddw          %%mm2,                  %%mm5\n"  // lum1 +green 00 G6 00 G4 00 G2 00 G0
            "psraw          $6,                     %%mm4\n"  // R1 0 .. 64
            "movq           %%mm7,                  %%mm3\n"  // lum2                       00 L7 00 L5 00 L3 00 L1
            "psraw          $6,                     %%mm5\n"  // G1  - .. +
            "paddw          %%mm0,                  %%mm7\n"  // Lum2 +blue 00 B7 00 B5 00 B3 00 B1
            "psraw          $6,                     %%mm6\n"  // B1         0 .. 64
            "packuswb       %%mm4,                  %%mm4\n"  // R1 R1
            "packuswb       %%mm5,                  %%mm5\n"  // G1 G1
            "packuswb       %%mm6,                  %%mm6\n"  // B1 B1
            "punpcklbw      %%mm4,                  %%mm4\n"
            "punpcklbw      %%mm5,                  %%mm5\n"

            "pand           mpeg2_MMX_redmask,      %%mm4\n"
            "psllw          $3,                     %%mm5\n"  // GREEN       1
            "punpcklbw      %%mm6,                  %%mm6\n"
            "pand           mpeg2_MMX_grnmask,      %%mm5\n"
            "pand           mpeg2_MMX_redmask,      %%mm6\n"
            "por            %%mm5,                  %%mm4\n"  //
            "psrlw          $11,                    %%mm6\n"  // BLUE		 1
            "movq           %%mm3,                  %%mm5\n"  // lum2
            "paddw          %%mm1,                  %%mm3\n"  // lum2 +red	 00 R7 00 R5 00 R3 00 R1
            "paddw          %%mm2,                  %%mm5\n"  // lum2 +green 00 G7 00 G5 00 G3 00 G1
            "psraw          $6,                     %%mm3\n"  // R2
            "por            %%mm6,                  %%mm4\n"  // MM4
            "psraw          $6,                     %%mm5\n"  // G2
            "movq           (%2, %3),               %%mm6\n"  // L3
            "psraw          $6,                     %%mm7\n"
            "packuswb       %%mm3,                  %%mm3\n"
            "packuswb       %%mm5,                  %%mm5\n"
            "packuswb       %%mm7,                  %%mm7\n"
            "pand           mpeg2_MMX_00FFw,        %%mm6\n"  // L3
            "punpcklbw      %%mm3,                  %%mm3\n"
            "punpcklbw      %%mm5,                  %%mm5\n"
            "pmullw         mpeg2_MMX_Ycoeff,       %%mm6\n"  // lum3
            "punpcklbw      %%mm7,                  %%mm7\n"
            "psllw          $3,                     %%mm5\n"  // GREEN 2
            "pand           mpeg2_MMX_redmask,      %%mm7\n"
            "pand           mpeg2_MMX_redmask,      %%mm3\n"
            "psrlw          $11,                    %%mm7\n"  // BLUE  2
            "pand           mpeg2_MMX_grnmask,      %%mm5\n"
            "por            %%mm7,  				%%mm3\n"
            "movq           (%2,%3),				%%mm7\n"  // L4
            "por            %%mm5,  				%%mm3\n"	 //
            "psrlw          $8,                     %%mm7\n"    // L4
            "movq           %%mm4,  				%%mm5\n"
            "punpcklwd      %%mm3,                  %%mm4\n"
            "pmullw         mpeg2_MMX_Ycoeff,       %%mm7\n"    // lum4
            "punpckhwd      %%mm3,                  %%mm5\n"

            "movq           %%mm4,  				(%4)\n"
            "movq           %%mm5,  				8(%4)\n"

            "movq           %%mm6,  				%%mm4\n"		// Lum3
            "paddw          %%mm0,                  %%mm6\n"                // Lum3 +blue

            "movq           %%mm4,  				%%mm5\n"						// Lum3
            "paddw          %%mm1,                  %%mm4\n"       // Lum3 +red
            "paddw          %%mm2,                  %%mm5\n"                        // Lum3 +green
            "psraw          $6, 				    %%mm4\n"
            "movq           %%mm7,   			    %%mm3\n"	// Lum4
            "psraw          $6, 				    %%mm5\n"
            "paddw          %%mm0,                  %%mm7\n"                   // Lum4 +blue
            "psraw          $6,                     %%mm6\n"                        // Lum3 +blue
            "movq           %%mm3,                  %%mm0\n"  // Lum4
            "packuswb       %%mm4,                  %%mm4\n"
            "paddw          %%mm1,                  %%mm3\n"  // Lum4 +red
            "packuswb       %%mm5,                  %%mm5\n"
            "paddw          %%mm2,                  %%mm0\n"         // Lum4 +green
            "packuswb       %%mm6,                  %%mm6\n"
            "punpcklbw      %%mm4,                  %%mm4\n"
            "punpcklbw      %%mm5,                  %%mm5\n"
            "punpcklbw      %%mm6,                  %%mm6\n"
            "psllw          $3,                     %%mm5\n" // GREEN 3
            "pand           mpeg2_MMX_redmask,      %%mm4\n"
            "psraw          $6,         			%%mm3\n" // psr 6
            "psraw          $6,         			%%mm0\n"
            "pand           mpeg2_MMX_redmask,      %%mm6\n" // BLUE
            "pand           mpeg2_MMX_grnmask,      %%mm5\n"
            "psrlw          $11,                    %%mm6\n"  // BLUE  3
            "por            %%mm5,  				%%mm4\n"
            "psraw          $6,                     %%mm7\n"
            "por            %%mm6,      			%%mm4\n"
            "packuswb       %%mm3,                  %%mm3\n"
            "packuswb       %%mm0,                  %%mm0\n"
            "packuswb       %%mm7,                  %%mm7\n"
            "punpcklbw      %%mm3,                  %%mm3\n"
            "punpcklbw      %%mm0,                  %%mm0\n"
            "punpcklbw      %%mm7,                  %%mm7\n"
            "pand                   mpeg2_MMX_redmask,    %%mm3\n"
            "pand                   mpeg2_MMX_redmask,    %%mm7\n" // BLUE
            "psllw          $3,                             %%mm0\n" // GREEN 4
            "psrlw          $11,                            %%mm7\n"
            "pand                   mpeg2_MMX_grnmask,    %%mm0\n"
            "por                    %%mm7,                  %%mm3\n"
            "addl                   $8,                             %6\n"
            "por                    %%mm0,                  %%mm3\n"

            "movq                   %%mm4,                  %%mm5\n"

            "punpcklwd      %%mm3,                  %%mm4\n"
            "punpckhwd      %%mm3,                  %%mm5\n"

            "movq                   %%mm4,                  (%4,%5,2)\n"
            "movq                   %%mm5,                  8(%4,%5,2)\n"

            "addl                   $8,                             %2\n"
            "addl                   $4,                             %0\n"
            "addl                   $4,                             %1\n"
            "cmpl                   %3,                             %6\n"
            "leal                   16(%4),                 %4\n"
        "jl             1b\n"
        "addl           %3,     %2\n"                   /* lum += cols */
        "addl           %7,     %4\n"                   /* row1 += mod */
        "movl           $0,     %6\n"
        "cmpl           %8,     %2\n"
        "jl             1b\n"
        : : "r" (cr), 
			"r" (cb), 
			"r" (lum), 
			"r" (cols), 
			"r" (row1) ,
			"r" (col1), 
			"m" (x), 
			"m" (mod), 
			"m" (y)
		);
}

static unsigned long long  mpeg2_MMX_U_80    = 0x0000008000800000;
static unsigned long long  mpeg2_MMX_V_80    = 0x0000000000800080;
static long long  mpeg2_MMX_U_COEF    = 0x00000058ffd30000;
static long long  mpeg2_MMX_V_COEF    = 0x00000000ffea006f;

inline void mpeg2_bgra32_mmx(unsigned long y, 
		unsigned long u, 
		unsigned long v, 
		unsigned long *output)
{
asm("
// Output will be 0x00rrggbb with the 00 trailing so this can also be used
// for bgr24.
	movd (%0), %%mm0;          // Load y   0x00000000000000yy
	movd (%1), %%mm1;          // Load u    0x00000000000000cr
	movq %%mm0, %%mm3;         // Copy y to temp
	psllq $16, %%mm1;          // Shift u   0x0000000000cr0000
	movd (%2), %%mm2;          // Load v    0x00000000000000cb
	psllq $16, %%mm3;          // Shift y
	movq %%mm1, %%mm4;         // Copy u to temp
	por %%mm3, %%mm0;          // Overlay new y byte 0x0000000000yy00yy
	psllq $16, %%mm4;          // Shift u
	movq %%mm2, %%mm5;         // Copy v to temp
	psllq $16, %%mm3;          // Shift y 
	por %%mm4, %%mm1;          // Overlay new u byte 0x000000cr00cr0000
	psllq $16, %%mm5;          // Shift v 
	por %%mm3, %%mm0;          // Overlay new y byte 0x000000yy00yy00yy
	por %%mm5, %%mm2;          // Overlay new v byte 0x0000000000cb00cb

// mm0: 0x000000yy00yy00yy mm1: 0x000000uu00uu0000 mm2: 0x0000000000vv00vv
 	psubw mpeg2_MMX_U_80, %%mm1;    // Subtract 128 from u 0x000000uu00uu0000
 	pmullw mpeg2_MMX_U_COEF, %%mm1; // Multiply u coeffs 0x0000uuuuuuuu0000
 	psllw $6, %%mm0;                // Shift y coeffs 0x0000yyy0yyy0yyy0
 	psubw mpeg2_MMX_V_80, %%mm2;    // Subtract 128 from v 0x0000000000cb00cb
 	pmullw mpeg2_MMX_V_COEF, %%mm2; // Multiply v coeffs 0x0000crcrcrcrcrcr

// mm0: 0x000000yy00yy00yy mm1: 0x0000uuuuuuuu0000 mm2: 0x00000000vvvvvvvv
	paddw %%mm1, %%mm0;        // Add u to result
	paddw %%mm2, %%mm0;        // Add v to result 0x0000rrrrggggbbbb
	psraw $6, %%mm0;           // Demote precision
	packuswb %%mm0, %%mm0;     // Pack into ARGB 0x0000000000rrggbb
	movd %%mm0, (%3);          // Store output
	"
:
: "r" (&y), "r" (&u), "r" (&v), "r" (output));
}

#endif

#define DITHER_ROW_HEAD \
	for(h = 0; h < video->output_h; h++) \
	{ \
		y_in = &src[0][video->y_table[h] * video->coded_picture_width]; \
		cb_in = &src[1][video->y_table[h] / 2 * video->chrom_width]; \
		cr_in = &src[2][video->y_table[h] / 2 * video->chrom_width]; \
		data = output_rows[h];

#define DITHER_ROW_TAIL \
	}

#define DITHER_SCALE_HEAD \
	for(w = 0; w < video->output_w; w++) \
	{ \
		uv_subscript = video->x_table[w] / 2; \
		y_l = y_in[video->x_table[w]]; \
		y_l <<= 16; \
		r_l = (y_l + video->cr_to_r[cr_in[uv_subscript]]) >> 16; \
		g_l = (y_l + video->cr_to_g[cr_in[uv_subscript]] + video->cb_to_g[cb_in[uv_subscript]]) >> 16; \
		b_l = (y_l + video->cb_to_b[cb_in[uv_subscript]]) >> 16;

#define DITHER_SCALE_TAIL \
	}

#define DITHER_HEAD \
    for(w = 0; w < video->horizontal_size; w++) \
	{ \
		y_l = *y_in++; \
		y_l <<= 16; \
		r_l = (y_l + video->cr_to_r[*cr_in]) >> 16; \
		g_l = (y_l + video->cr_to_g[*cr_in] + video->cb_to_g[*cb_in]) >> 16; \
		b_l = (y_l + video->cb_to_b[*cb_in]) >> 16;

#define DITHER_TAIL \
    	if(w & 1) \
		{ \
        	cr_in++; \
        	cb_in++; \
    	} \
    }


#define STORE_PIXEL_BGR888 \
	*data++ = CLIP(b_l); \
	*data++ = CLIP(g_l); \
	*data++ = CLIP(r_l);

#define STORE_PIXEL_BGRA8888 \
	*data++ = CLIP(b_l); \
	*data++ = CLIP(g_l); \
	*data++ = CLIP(r_l); \
	*data++ = 0;

#define STORE_PIXEL_RGB565 \
	*((unsigned short*)data)++ = \
		((CLIP(r_l) & 0xf8) << 8) | \
		((CLIP(g_l) & 0xfc) << 3) | \
		((CLIP(b_l) & 0xf8) >> 3);

#define STORE_PIXEL_RGB888 \
	*data++ = CLIP(r_l); \
	*data++ = CLIP(g_l); \
	*data++ = CLIP(b_l);



// Only good for YUV 4:2:0
int mpeg2video_ditherframe(mpeg2video_t *video, unsigned char *src[], unsigned char **output_rows)
{
	int h = 0;
	register unsigned char *y_in, *cb_in, *cr_in;
	long y_l, r_l, b_l, g_l;
	register unsigned char *data;
	register int uv_subscript, step, w = -1;

#ifdef HAVE_MMX
	if(video->color_model == MPEG2_RGB565 && 
		video->output_w == video->horizontal_size &&
		video->output_h == video->vertical_size)
	{
// MMX is available for RGB565 unscaled
		mpeg2video_rgb16_mmx(src[0], 
			src[2], 
			src[1], 
			output_rows[0], 
			video->output_h, 
			video->output_w, 
			(output_rows[1] - output_rows[0]) / 2 - video->output_w);
	}
	else
	if(video->color_model == MPEG2_BGRA8888 || video->color_model == MPEG2_BGR888)
	{
		if(video->color_model == MPEG2_BGRA8888) step = 4;
		else
		if(video->color_model == MPEG2_BGR888) step = 3;

		DITHER_ROW_HEAD
// Transfer row with scaling
			if(video->output_w != video->horizontal_size)
			{
				for(w = 0; w < video->output_w; w++)
				{
					uv_subscript = video->x_table[w] / 2;
					mpeg2_bgra32_mmx(y_in[video->x_table[w]], 
						cr_in[uv_subscript], 
						cb_in[uv_subscript], 
						(unsigned long*)data);
					data += step;
				}
			}
			else
// Transfer row unscaled
			{
				for(w = 0; w < video->output_w; w += 2)
				{
					mpeg2_bgra32_mmx(*y_in++, 
						*cr_in, 
						*cb_in, 
						(unsigned long*)data);
					data += step;
					mpeg2_bgra32_mmx(*y_in++, 
						*cr_in, 
						*cb_in, 
						(unsigned long*)data);
					data += step;
        			cr_in++;
        			cb_in++;
				}
			}
		DITHER_ROW_TAIL
	}
	else
#endif
	{
		DITHER_ROW_HEAD
// Transfer row with scaling
			if(video->output_w != video->horizontal_size)
			{
				switch(video->color_model)
				{
					case MPEG2_BGR888:
						DITHER_SCALE_HEAD
						STORE_PIXEL_BGR888
						DITHER_SCALE_TAIL
						break;
					case MPEG2_BGRA8888:
						DITHER_SCALE_HEAD
						STORE_PIXEL_BGRA8888
						DITHER_SCALE_TAIL
						break;
					case MPEG2_RGB565:
						DITHER_SCALE_HEAD
						STORE_PIXEL_RGB565
						DITHER_SCALE_TAIL
						break;
					case MPEG2_RGB888:
						DITHER_SCALE_HEAD
						STORE_PIXEL_RGB888
						DITHER_SCALE_TAIL
						break;
				}
			}
			else
			{
// Transfer row unscaled
				switch(video->color_model)
				{
					case MPEG2_BGR888:
						DITHER_HEAD
						STORE_PIXEL_BGR888
						DITHER_TAIL
						break;
					case MPEG2_BGRA8888:
						DITHER_HEAD
						STORE_PIXEL_BGRA8888
						DITHER_TAIL
						break;
					case MPEG2_RGB565:
						DITHER_HEAD
						STORE_PIXEL_RGB565
						DITHER_TAIL
						break;
					case MPEG2_RGB888:
						DITHER_HEAD
						STORE_PIXEL_RGB888
						DITHER_TAIL
						break;
				}
			}
		DITHER_ROW_TAIL
	} // End of non-MMX
	return 0;
}

int mpeg2video_ditherframe444(mpeg2video_t *video, unsigned char *src[])
{
	return 0;
}

int mpeg2video_dithertop(mpeg2video_t *video, unsigned char *src[])
{
	return mpeg2video_ditherframe(video, src, video->output_rows);
}

int mpeg2video_dithertop444(mpeg2video_t *video, unsigned char *src[])
{
	return 0;
}

int mpeg2video_ditherbot(mpeg2video_t *video, unsigned char *src[])
{
	return 0;
}

int mpeg2video_ditherbot444(mpeg2video_t *video, unsigned char *src[])
{
	return 0;
}

int mpeg2video_dither(mpeg2video_t *video, unsigned char *src[])
{
	video->last_src = src;
// Don't want to put in the output while seeking.
	if(!video->use_output) return 0;

// Copy the frame to the output with YUV to RGB conversion
  	if(video->prog_seq)
	{
    	if(video->chroma_format != CHROMA444)
		{
    		mpeg2video_ditherframe(video, src, video->output_rows);
    	}
    	else
    	  	mpeg2video_ditherframe444(video, src);
  	}
	else
	{
   		if((video->pict_struct == FRAME_PICTURE && video->topfirst) || 
			video->pict_struct == BOTTOM_FIELD)
		{
/* top field first */
    		if (video->chroma_format != CHROMA444)
			{
        		mpeg2video_dithertop(video, src);
        		mpeg2video_ditherbot(video, src);
    		}
    		else 
			{
        		mpeg2video_dithertop444(video, src);
        		mpeg2video_ditherbot444(video, src);
    		}
    	}
    	else 
		{
/* bottom field first */
    		if(video->chroma_format != CHROMA444)
			{
        		mpeg2video_ditherbot(video, src);
        		mpeg2video_dithertop(video, src);
    		}
    		else 
			{
        		mpeg2video_ditherbot444(video, src);
        		mpeg2video_dithertop444(video, src);
    		}
    	}
  	}
	return 0;
}

int mpeg2video_display_second_field(mpeg2video_t *video)
{
// Not used
	return 0;
}
