#include "asset.h"
#include "file.h"
#include "filemov.h"
#include "filempeg.h"


File::File()
{
	file = 0;
	asset = 0;
}

File::~File()
{
	close_file();
}

int File::set_processors(int cpus)
{
	this->cpus = cpus;
}

int File::set_prebuffer(long size)
{
	this->prebuffer_size = size;
}

int File::try_to_open_file(Asset *asset)
{
	this->asset = asset;

// get the format now
	FILE *stream;
	if(!(stream = fopen(asset->path, "rb")))
	{
// file not found
		return 1;
	}

// ============================== Check formats
// add new formats here
	if(mpeg2_check_sig(asset->path))
	{
		fclose(stream);
		file = new FileMPEG(asset, this);
	}
	else
	if(quicktime_check_sig(asset->path))
	{
// MOV file
// should be last because quicktime lacks a magic number
		fclose(stream);
		file = new FileMOV(asset, this);
	}
	else
	{
		fclose(stream);
		return 2;
	}

	int result = file->open_file();
	total_frames = get_video_length();
	total_samples = get_audio_length();
	if(result)
	{
// failure
		delete file;
		file = 0;
	}
	return result;
}

int File::close_file()
{
	if(file) 
	{
		file->close_file();
		delete file;
	}

	file = 0;
	asset = 0;
}

long File::get_audio_length() { return file->get_audio_length(); }

long File::get_video_length() { return file->get_video_length(); }

long File::get_video_position() { return file->get_video_position(); }

long File::get_audio_position() { return file->get_audio_position(); }

int File::set_audio_position(long x) 
{
	file->set_audio_position(x); 
}

int File::set_video_position(long x) 
{ 
	if(x < total_frames)
		file->set_video_position(x);
	else
		file->set_video_position(total_frames - 1);
}

int File::load_into_ram() { file->load_into_ram(); }

int File::lock_read()
{
	read_lock.lock();
}

int File::unlock_read()
{
	read_lock.unlock();
}

int File::read_raw(char *buffer, long len)
{
	int result = 0;
	if(file) result = file->read_raw(buffer, len);
	return result;
}

int File::frame_buffer_copy_possible(int color_model)
{
	if(file) return file->frame_buffer_copy_possible(color_model);
	return 0;
}

int File::read_frame_buffer(unsigned char **frame_buffer, int w, int h, int bitmap_color_model)
{
	return file->read_frame_buffer(frame_buffer, w, h, bitmap_color_model);
}

int File::read_frame(unsigned char *frame)
{
	int result = 0;
	if(file) result = file->read_frame(frame);
	return result;
}
