#ifndef FILEBASE_H
#define FILEBASE_H

#include "asset.inc"
#include "file.inc"
#include "sizes.h"

// inherited by every file interpreter
class FileBase
{
public:
	FileBase(Asset *asset, File *file);
	virtual ~FileBase();

	int close_file();
	int reset_parameters();

	virtual int reset_parameters_derived() { return 0; };
	virtual int read_header() { return 0; };
	virtual int open_file() { return 0; };
	virtual int close_file_derived() { return 0; };
	virtual long get_audio_length() { return 0; };
	virtual long get_video_length() { return 0; };
	virtual long get_video_position() { return 0; };
	virtual long get_audio_position() { return 0; };
	virtual int set_video_position(long x) { return 0; };
	virtual int set_audio_position(long x) { return 0; };
	virtual int read_raw(char *buffer, long len) { return 0; };
	virtual int read_frame_buffer(unsigned char **frame_buffer, int w, int h, int bitmap_color_model) { return 0; };
	virtual int frame_buffer_copy_possible(int color_model) { return 0; };
	virtual int read_frame(unsigned char *frame) { return 0; };
	virtual int load_into_ram() {};
	
	File *file;

protected:
	int get_read_buffer(long len);

	int internal_byte_order;
	int match4(char *in, char *out);
	int delete_cache();
	Asset *asset;
// for loading into RAM
	char **ram_cache;
	TWO *read_buffer; // Temporary storage for audio output
	long read_size;   // Sample size of temporary storage
	long cached_frames;
};

#endif
