#include "asset.h"
#include "bcbase.h"
#include "file.h"
#include "filempeg.h"
#include "vframe.h"



FileMPEG::FileMPEG(Asset *asset, File *file)
 : FileBase(asset, file)
{
	reset_parameters();
	asset->format = MPEG;
	asset->byte_order = 0;
}

FileMPEG::~FileMPEG()
{
	close_file();
}

int FileMPEG::reset_parameters_derived()
{
	file = 0;
}

int FileMPEG::open_file()
{
	if(!(file = mpeg2_open(asset->path)))
	{
		perror("FileMPEG::open_file");
		return 1;
	}

	mpeg2_set_cpus(file, FileBase::file->cpus);
	read_header();
	
	if(!asset->video_data && !asset->audio_data)
	{
		mpeg2_close(file);
		return 3;
	}
	return 0;
}

int FileMPEG::close_file_derived()
{
	if(file) mpeg2_close(file);
	file = 0;
}

int FileMPEG::read_header()
{
// determine if the audio can be read before declaring audio data
	if(mpeg2_has_audio(file)) asset->audio_data = 1;

	if(asset->audio_data)
	{
		asset->channels = mpeg2_audio_channels(file);
		asset->rate = mpeg2_sample_rate(file);
		asset->bits = 16;
		asset->signed_ = 1;
		asset->byte_order = internal_byte_order;
	}

// determine if the video can be read before declaring video data
	if(mpeg2_has_video(file)) asset->video_data = 1;

	if(asset->video_data)
	{
		asset->layers = 1;
		asset->width = mpeg2_video_width(file);
		asset->height = mpeg2_video_height(file);
		asset->frame_rate = mpeg2_frame_rate(file);
		asset->compression[0] = 0;
	}
	if(asset->channels > 2) asset->channels = 2;
}

long FileMPEG::get_audio_length()
{
	long result = mpeg2_audio_samples(file);
	return result;
}

long FileMPEG::get_video_length()
{
	long result = mpeg2_video_frames(file);
	return result;
}

long FileMPEG::get_video_position()
{
	return mpeg2_get_frame(file);
}

long FileMPEG::get_audio_position()
{
	return mpeg2_get_sample(file);
}

int FileMPEG::set_audio_position(long x)
{
	audio_position = x;
	return mpeg2_set_sample(file, x);
}

int FileMPEG::set_video_position(long x)
{
	video_position = x;
	return mpeg2_set_frame(file, x);
}

int FileMPEG::frame_buffer_copy_possible(int color_model)
{
	if(color_model == BC_BITMAP_RGB565 ||
		color_model == BC_BITMAP_BGR888 ||
		color_model == BC_BITMAP_BGRA8888) 
		return 1;
	else
		return 0;
}

int FileMPEG::translate_color_model(int color_model)
{
	switch(color_model)
	{
		case BC_BITMAP_RGB565:   return MPEG2_RGB565;   break;
		case BC_BITMAP_BGR888:   return MPEG2_BGR888;   break;
		case BC_BITMAP_BGRA8888: return MPEG2_BGRA8888; break;
	}
	return -1;
}

int FileMPEG::read_frame_buffer(unsigned char **frame_buffer, int w, int h, int bitmap_color_model)
{
	int result;
	result = mpeg2_read_frame(file, frame_buffer, w, h, translate_color_model(bitmap_color_model));
	video_position++;
	return result;
}

int FileMPEG::read_frame(unsigned char *frame)
{
	int result, i;
	unsigned char **row_pointers;
	row_pointers = new unsigned char*[asset->height];
	for(i = 0; i < asset->height; i++)
		row_pointers[i] = &frame[i * asset->width * 3];

	result = mpeg2_read_frame(file, row_pointers, asset->width, asset->height, MPEG2_RGB888);
	delete row_pointers;
	video_position++;
	return result;
}

int FileMPEG::read_raw(char *buffer, long len)
{
	int channel;
	int i, j;
	int result = 0;

	get_read_buffer(len);

	for(channel = 0; channel < asset->channels && !result; channel++)
	{
		mpeg2_set_sample(file, audio_position);
		result = mpeg2_read_audio(file, 0, read_buffer, channel, len);

		TWO *output_ptr = (TWO*)buffer + channel;
		TWO *input_ptr = (TWO*)read_buffer;
		TWO *input_end = input_ptr + len;

// Interlace the audio for the sound driver.
		while(input_ptr < input_end)
		{
			*output_ptr = *input_ptr++;
			output_ptr += asset->channels;
		}
	}

	audio_position += len;
	return 0;
}

int FileMPEG::load_into_ram()
{
	if(asset->video_data)
	{
//		quicktime_cache_frames(file, 0);
	}
}
