#ifndef MWINDOW_H
#define MWINDOW_H

#define DEFAULTW  320
#define DEFAULTH  160
#define BUTTONW 15
#define BUTTONH 15
#define TOTAL_LOADS 5

#include "arraylist.h"
#include "settings.inc"
#include "asset.inc"
#include "bcbase.h"
#include "defaults.inc"
#include "file.inc"
#include "mainwindow.inc"
#include "mutex.h"
#include "playbackengine.inc"
#include "thread.h"

class Load;
class LoadPrevious;
class LoadThread;
class MWindowGUI;
class MainMenu;
class MainCanvas;
class MainScrollbar;
class PlayButton;
class ErrorThread;
class FrameBackButton;
class FrameForwardButton;
class SettingsMenu;


class MainWindow
{
public:
	MainWindow(ArrayList<char*> *playlist);
	~MainWindow();

	int create_objects();
	int run_program();
	int exit_cleanly();
	int reset_parameters();
	int fix_size(int &w, int &h, float zoom_factor);
	float get_aspect_ratio();

// =================== all operations start here
	int load_file(char *path);
	int close_file();
	int start_playback();
	int stop_playback();
	int frame_forward();
	int frame_backward();

// flash a frame from any point in the movie on the screen
	int flash_frame(long frame_number);

// Playback
	int arm_frame(long frame_number);
	int flash_armed_frame();

	long total_samples;
	long samplerate;
	float frame_rate;
	long current_sample;
	int every_frame;
	int smp;
//	int load_into_memory;
	int software_sync;
	long prebuffer_size;

	ErrorThread *error_thread;
	Defaults *defaults;
	ArrayList<char*> *playlist;
	MWindowGUI *gui;
	Asset *asset;   // description of file or 0 if no file loaded
	File *file;   // 0 when no file loaded otherwise pointer to open file
	PlaybackEngine *engine;
	SettingsMenu *settingsmenu;
	char default_path[1024];
	float aspect_w, aspect_h;
	int square_pixels; // Ignore aspect ratio if 1
	float actual_framerate; // Actual framerate being achieved.
private:
// get length of file from file
	long get_length();
	unsigned char *frame;
};

class ErrorThread : public Thread
{
public:
	ErrorThread();
	~ErrorThread();

	int run_error(char *text1, char *text2 = 0, char *text3 = 0, char *text4 = 0);
	void run();
	Mutex startup_lock;
	char *text1;
	char *text2;
	char *text3;
	char *text4;
};

class MWindowGUI : public BC_Window
{
public:
	MWindowGUI(MainWindow *mwindow, int w, int h);
	~MWindowGUI();

	int create_objects();
	int close_event();
// Use size of canvas from loaded movie file.
	int resize_canvas(int w, int h);
// Use size of window from resize event.
	int resize_event(int w, int h);
// Fix size of scrollbar for movie length and position.
	int resize_scrollbar();
// Make scrollbar reflect current position
	int update_scrollbar();
	int flash_frame(unsigned char *frame);

	float zoom_factor;

	MainWindow *mwindow;
	MWindowGUI *gui;
	MainMenu *menu;
	MainCanvas *canvas;
	MainScrollbar *scrollbar;
	PlayButton *playbutton;
	FrameBackButton *backbutton;
	FrameForwardButton *forwardbutton;

	int resize_widgets();
	BC_Bitmap *bitmap;
};

class MainMenu : public BC_MenuBar
{
public:
	MainMenu(MainWindow *mwindow, int w);
	~MainMenu();

	int create_objects();

// most recent loads
	int add_load(char *path);
	int init_loads(Defaults *defaults);
	int save_loads(Defaults *defaults);
	int total_loads;

// for previous document loader
	Load *load_file;
	LoadPrevious *load_previous[TOTAL_LOADS];
	BC_Menu *filemenu;
	MainWindow *mwindow;
};

class Quit : public BC_MenuItem
{
public:
	Quit(MainWindow *mwindow);
	~Quit();
	int handle_event();
	MainWindow *mwindow;
};

class Load : public BC_MenuItem
{
public:
	Load(MainWindow *mwindow);
	~Load();
	int handle_event();
	MainWindow *mwindow;
	LoadThread *thread;
};

// class LoadIntoMemory : public BC_MenuItem
// {
// public:
// 	LoadIntoMemory(MainWindow *mwindow);
// 	~LoadIntoMemory();
// 	int handle_event();
// 	MainWindow *mwindow;
// };

class SoftwareSync : public BC_MenuItem
{
public:
	SoftwareSync(MainWindow *mwindow);
	~SoftwareSync();
	int handle_event();
	MainWindow *mwindow;
};

class OriginalSize : public BC_MenuItem
{
public:
	OriginalSize(MainWindow *mwindow);
	~OriginalSize();
	int handle_event();
	MainWindow *mwindow;
};

class SettingsMenu : public BC_MenuItem
{
public:
	SettingsMenu(MainWindow *mwindow);
	~SettingsMenu();
	int handle_event();
	MainWindow *mwindow;
	SettingsThread *thread;
};

class EveryFrame : public BC_MenuItem
{
public:
	EveryFrame(MainWindow *mwindow);
	~EveryFrame();
	int handle_event();
	MainWindow *mwindow;
};

class LoadPrevious : public BC_MenuItem
{
public:
	LoadPrevious(Load *loadfile);
	int handle_event();
	void run();

	int set_path(char *path);

	Load *loadfile;
	char path[1024];
};

class Close : public BC_MenuItem
{
public:
	Close(MainWindow *mwindow);
	~Close();
	int handle_event();
	MainWindow *mwindow;
};


class MainCanvas : public BC_Canvas
{
public:
	MainCanvas(MainWindow *mwindow, int x, int y, int w, int h);
	~MainCanvas();

	int notify_load();
	int unnotify_load();

	MainWindow *mwindow;
};

class MainScrollbar : public BC_ISlider
{
public:
	MainScrollbar(MainWindow *mwindow, int x, int y, int w);
	~MainScrollbar();
	int handle_event();
	MainWindow *mwindow;
};

class PlayButton : public BC_ForwardButton
{
public:
	PlayButton(MainWindow *mwindow, int x, int y);
	~PlayButton();
	int handle_event();
	int keypress_event();

	MainWindow *mwindow;
};

class FrameBackButton : public BC_LeftTriangleButton
{
public:
	FrameBackButton(MainWindow *mwindow, int x, int y);
	~FrameBackButton();
	int handle_event();
	int keypress_event();
	MainWindow *mwindow;
};

class FrameForwardButton : public BC_RightTriangleButton
{
public:
	FrameForwardButton(MainWindow *mwindow, int x, int y);
	~FrameForwardButton();
	int handle_event();
	int keypress_event();
	MainWindow *mwindow;
};

class LoadThread : public Thread
{
public:
	LoadThread(MainWindow *mwindow);
	~LoadThread();
	void run();
	MainWindow *mwindow;
};



class LoadDialog : public BC_FileBox
{
public:
	LoadDialog(MainWindow *mwindow);
	~LoadDialog();

	int ok_event();
	int cancel_event();
	MainWindow *mwindow;
};

#endif
